package Mozilla::PublicSuffix;

use strict;
use warnings FATAL => 'all';
use utf8;
use Exporter qw(import);
use URI::_idna;

our @EXPORT_OK = qw(public_suffix);

our $VERSION = 'v0.1.19'; # VERSION
# ABSTRACT: Get a domain name's public suffix via the Mozilla Public Suffix List

my $dn_re = do {
    my $aln = '[[:alnum:]]';
    my $anh = '[[:alnum:]-]';
    my $re_str = "(?:$aln(?:(?:$anh){0,61}$aln)?"
               . "(?:\\.$aln(?:(?:$anh){0,61}$aln)?)*)";

    qr/^$re_str$/;
};
sub public_suffix {
    my $string = shift;

    # Decode domains in punycode form:
    my $domain = defined($string) && ref($string) eq ''
        ? index($string, 'xn--') == -1
            ? lc $string
            : eval { lc URI::_idna::decode($string) }
        : '';

    # Search using the full domain and a substring consisting of its lowest
    # levels, or return early (undef in scalar context) if the domain name is
    # not well-formed according to RFC 1123:
    return $domain =~ $dn_re ? _find_rule($domain) : ( );
}

my %rules = qw(ac i com.ac i edu.ac i gov.ac i net.ac i mil.ac i org.ac i ad i nom.ad i ae i co.ae i net.ae i org.ae i sch.ae i ac.ae i gov.ae i mil.ae i aero i accident-investigation.aero i accident-prevention.aero i aerobatic.aero i aeroclub.aero i aerodrome.aero i agents.aero i aircraft.aero i airline.aero i airport.aero i air-surveillance.aero i airtraffic.aero i air-traffic-control.aero i ambulance.aero i amusement.aero i association.aero i author.aero i ballooning.aero i broker.aero i caa.aero i cargo.aero i catering.aero i certification.aero i championship.aero i charter.aero i civilaviation.aero i club.aero i conference.aero i consultant.aero i consulting.aero i control.aero i council.aero i crew.aero i design.aero i dgca.aero i educator.aero i emergency.aero i engine.aero i engineer.aero i entertainment.aero i equipment.aero i exchange.aero i express.aero i federation.aero i flight.aero i freight.aero i fuel.aero i gliding.aero i government.aero i groundhandling.aero i group.aero i hanggliding.aero i homebuilt.aero i insurance.aero i journal.aero i journalist.aero i leasing.aero i logistics.aero i magazine.aero i maintenance.aero i marketplace.aero i media.aero i microlight.aero i modelling.aero i navigation.aero i parachuting.aero i paragliding.aero i passenger-association.aero i pilot.aero i press.aero i production.aero i recreation.aero i repbody.aero i res.aero i research.aero i rotorcraft.aero i safety.aero i scientist.aero i services.aero i show.aero i skydiving.aero i software.aero i student.aero i taxi.aero i trader.aero i trading.aero i trainer.aero i union.aero i workinggroup.aero i works.aero i af i gov.af i com.af i org.af i net.af i edu.af i ag i com.ag i org.ag i net.ag i co.ag i nom.ag i ai i off.ai i com.ai i net.ai i org.ai i al i com.al i edu.al i gov.al i mil.al i net.al i org.al i am i an i com.an i net.an i org.an i edu.an i ao i ed.ao i gv.ao i og.ao i co.ao i pb.ao i it.ao i aq i ar i com.ar i edu.ar i gob.ar i gov.ar i int.ar i mil.ar i net.ar i org.ar i tur.ar i arpa i e164.arpa i in-addr.arpa i ip6.arpa i iris.arpa i uri.arpa i urn.arpa i as i gov.as i asia i at i ac.at i co.at i gv.at i or.at i au i com.au i net.au i org.au i edu.au i gov.au i asn.au i id.au i info.au i conf.au i oz.au i act.au i nsw.au i nt.au i qld.au i sa.au i tas.au i vic.au i wa.au i act.edu.au i nsw.edu.au i nt.edu.au i qld.edu.au i sa.edu.au i tas.edu.au i vic.edu.au i wa.edu.au i qld.gov.au i sa.gov.au i tas.gov.au i vic.gov.au i wa.gov.au i aw i com.aw i ax i az i com.az i net.az i int.az i gov.az i org.az i edu.az i info.az i pp.az i mil.az i name.az i pro.az i biz.az i ba i org.ba i net.ba i edu.ba i gov.ba i mil.ba i unsa.ba i unbi.ba i co.ba i com.ba i rs.ba i bb i biz.bb i co.bb i com.bb i edu.bb i gov.bb i info.bb i net.bb i org.bb i store.bb i tv.bb i bd w be i ac.be i bf i gov.bf i bg i a.bg i b.bg i c.bg i d.bg i e.bg i f.bg i g.bg i h.bg i i.bg i j.bg i k.bg i l.bg i m.bg i n.bg i o.bg i p.bg i q.bg i r.bg i s.bg i t.bg i u.bg i v.bg i w.bg i x.bg i y.bg i z.bg i 0.bg i 1.bg i 2.bg i 3.bg i 4.bg i 5.bg i 6.bg i 7.bg i 8.bg i 9.bg i bh i com.bh i edu.bh i net.bh i org.bh i gov.bh i bi i co.bi i com.bi i edu.bi i or.bi i org.bi i biz i bj i asso.bj i barreau.bj i gouv.bj i bm i com.bm i edu.bm i gov.bm i net.bm i org.bm i bn w bo i com.bo i edu.bo i gov.bo i gob.bo i int.bo i org.bo i net.bo i mil.bo i tv.bo i br i adm.br i adv.br i agr.br i am.br i arq.br i art.br i ato.br i b.br i bio.br i blog.br i bmd.br i cim.br i cng.br i cnt.br i com.br i coop.br i ecn.br i eco.br i edu.br i emp.br i eng.br i esp.br i etc.br i eti.br i far.br i flog.br i fm.br i fnd.br i fot.br i fst.br i g12.br i ggf.br i gov.br i imb.br i ind.br i inf.br i jor.br i jus.br i leg.br i lel.br i mat.br i med.br i mil.br i mp.br i mus.br i net.br i nom.br i not.br i ntr.br i odo.br i org.br i ppg.br i pro.br i psc.br i psi.br i qsl.br i radio.br i rec.br i slg.br i srv.br i taxi.br i teo.br i tmp.br i trd.br i tur.br i tv.br i vet.br i vlog.br i wiki.br i zlg.br i bs i com.bs i net.bs i org.bs i edu.bs i gov.bs i bt i com.bt i edu.bt i gov.bt i net.bt i org.bt i bv i bw i co.bw i org.bw i by i gov.by i mil.by i com.by i of.by i bz i com.bz i net.bz i org.bz i edu.bz i gov.bz i ca i ab.ca i bc.ca i mb.ca i nb.ca i nf.ca i nl.ca i ns.ca i nt.ca i nu.ca i on.ca i pe.ca i qc.ca i sk.ca i yk.ca i gc.ca i cat i cc i cd i gov.cd i cf i cg i ch i ci i org.ci i or.ci i com.ci i co.ci i edu.ci i ed.ci i ac.ci i net.ci i go.ci i asso.ci i aéroport.ci i int.ci i presse.ci i md.ci i gouv.ci i ck w www.ck e cl i gov.cl i gob.cl i co.cl i mil.cl i cm i co.cm i com.cm i gov.cm i net.cm i cn i ac.cn i com.cn i edu.cn i gov.cn i net.cn i org.cn i mil.cn i 公司.cn i 网络.cn i 網絡.cn i ah.cn i bj.cn i cq.cn i fj.cn i gd.cn i gs.cn i gz.cn i gx.cn i ha.cn i hb.cn i he.cn i hi.cn i hl.cn i hn.cn i jl.cn i js.cn i jx.cn i ln.cn i nm.cn i nx.cn i qh.cn i sc.cn i sd.cn i sh.cn i sn.cn i sx.cn i tj.cn i xj.cn i xz.cn i yn.cn i zj.cn i hk.cn i mo.cn i tw.cn i co i arts.co i com.co i edu.co i firm.co i gov.co i info.co i int.co i mil.co i net.co i nom.co i org.co i rec.co i web.co i com i coop i cr i ac.cr i co.cr i ed.cr i fi.cr i go.cr i or.cr i sa.cr i cu i com.cu i edu.cu i org.cu i net.cu i gov.cu i inf.cu i cv i cw i com.cw i edu.cw i net.cw i org.cw i cx i gov.cx i cy w cz i de i dj i dk i dm i com.dm i net.dm i org.dm i edu.dm i gov.dm i do i art.do i com.do i edu.do i gob.do i gov.do i mil.do i net.do i org.do i sld.do i web.do i dz i com.dz i org.dz i net.dz i gov.dz i edu.dz i asso.dz i pol.dz i art.dz i ec i com.ec i info.ec i net.ec i fin.ec i k12.ec i med.ec i pro.ec i org.ec i edu.ec i gov.ec i gob.ec i mil.ec i edu i ee i edu.ee i gov.ee i riik.ee i lib.ee i med.ee i com.ee i pri.ee i aip.ee i org.ee i fie.ee i eg i com.eg i edu.eg i eun.eg i gov.eg i mil.eg i name.eg i net.eg i org.eg i sci.eg i er w es i com.es i nom.es i org.es i gob.es i edu.es i et w eu i fi i aland.fi i fj w fk w fm i fo i fr i com.fr i asso.fr i nom.fr i prd.fr i presse.fr i tm.fr i aeroport.fr i assedic.fr i avocat.fr i avoues.fr i cci.fr i chambagri.fr i chirurgiens-dentistes.fr i experts-comptables.fr i geometre-expert.fr i gouv.fr i greta.fr i huissier-justice.fr i medecin.fr i notaires.fr i pharmacien.fr i port.fr i veterinaire.fr i ga i gb i gd i ge i com.ge i edu.ge i gov.ge i org.ge i mil.ge i net.ge i pvt.ge i gf i gg i co.gg i net.gg i org.gg i gh i com.gh i edu.gh i gov.gh i org.gh i mil.gh i gi i com.gi i ltd.gi i gov.gi i mod.gi i edu.gi i org.gi i gl i gm i gn i ac.gn i com.gn i edu.gn i gov.gn i org.gn i net.gn i gov i gp i com.gp i net.gp i mobi.gp i edu.gp i org.gp i asso.gp i gq i gr i com.gr i edu.gr i net.gr i org.gr i gov.gr i gs i gt i com.gt i edu.gt i gob.gt i ind.gt i mil.gt i net.gt i org.gt i gu w gw i gy i co.gy i com.gy i net.gy i hk i com.hk i edu.hk i gov.hk i idv.hk i net.hk i org.hk i 公司.hk i 教育.hk i 敎育.hk i 政府.hk i 個人.hk i 个人.hk i 箇人.hk i 網络.hk i 网络.hk i 组織.hk i 網絡.hk i 网絡.hk i 组织.hk i 組織.hk i 組织.hk i hm i hn i com.hn i edu.hn i org.hn i net.hn i mil.hn i gob.hn i hr i iz.hr i from.hr i name.hr i com.hr i ht i com.ht i shop.ht i firm.ht i info.ht i adult.ht i net.ht i pro.ht i org.ht i med.ht i art.ht i coop.ht i pol.ht i asso.ht i edu.ht i rel.ht i gouv.ht i perso.ht i hu i co.hu i info.hu i org.hu i priv.hu i sport.hu i tm.hu i 2000.hu i agrar.hu i bolt.hu i casino.hu i city.hu i erotica.hu i erotika.hu i film.hu i forum.hu i games.hu i hotel.hu i ingatlan.hu i jogasz.hu i konyvelo.hu i lakas.hu i media.hu i news.hu i reklam.hu i sex.hu i shop.hu i suli.hu i szex.hu i tozsde.hu i utazas.hu i video.hu i id i ac.id i biz.id i co.id i desa.id i go.id i mil.id i my.id i net.id i or.id i sch.id i web.id i ie i gov.ie i il w im i ac.im i co.im i com.im i ltd.co.im i net.im i org.im i plc.co.im i tt.im i tv.im i in i co.in i firm.in i net.in i org.in i gen.in i ind.in i nic.in i ac.in i edu.in i res.in i gov.in i mil.in i info i int i eu.int i io i com.io i iq i gov.iq i edu.iq i mil.iq i com.iq i org.iq i net.iq i ir i ac.ir i co.ir i gov.ir i id.ir i net.ir i org.ir i sch.ir i ایران.ir i ايران.ir i is i net.is i com.is i edu.is i gov.is i org.is i int.is i it i gov.it i edu.it i abr.it i abruzzo.it i aosta-valley.it i aostavalley.it i bas.it i basilicata.it i cal.it i calabria.it i cam.it i campania.it i emilia-romagna.it i emiliaromagna.it i emr.it i friuli-v-giulia.it i friuli-ve-giulia.it i friuli-vegiulia.it i friuli-venezia-giulia.it i friuli-veneziagiulia.it i friuli-vgiulia.it i friuliv-giulia.it i friulive-giulia.it i friulivegiulia.it i friulivenezia-giulia.it i friuliveneziagiulia.it i friulivgiulia.it i fvg.it i laz.it i lazio.it i lig.it i liguria.it i lom.it i lombardia.it i lombardy.it i lucania.it i mar.it i marche.it i mol.it i molise.it i piedmont.it i piemonte.it i pmn.it i pug.it i puglia.it i sar.it i sardegna.it i sardinia.it i sic.it i sicilia.it i sicily.it i taa.it i tos.it i toscana.it i trentino-a-adige.it i trentino-aadige.it i trentino-alto-adige.it i trentino-altoadige.it i trentino-s-tirol.it i trentino-stirol.it i trentino-sud-tirol.it i trentino-sudtirol.it i trentino-sued-tirol.it i trentino-suedtirol.it i trentinoa-adige.it i trentinoaadige.it i trentinoalto-adige.it i trentinoaltoadige.it i trentinos-tirol.it i trentinostirol.it i trentinosud-tirol.it i trentinosudtirol.it i trentinosued-tirol.it i trentinosuedtirol.it i tuscany.it i umb.it i umbria.it i val-d-aosta.it i val-daosta.it i vald-aosta.it i valdaosta.it i valle-aosta.it i valle-d-aosta.it i valle-daosta.it i valleaosta.it i valled-aosta.it i valledaosta.it i vallee-aoste.it i valleeaoste.it i vao.it i vda.it i ven.it i veneto.it i ag.it i agrigento.it i al.it i alessandria.it i alto-adige.it i altoadige.it i an.it i ancona.it i andria-barletta-trani.it i andria-trani-barletta.it i andriabarlettatrani.it i andriatranibarletta.it i ao.it i aosta.it i aoste.it i ap.it i aq.it i aquila.it i ar.it i arezzo.it i ascoli-piceno.it i ascolipiceno.it i asti.it i at.it i av.it i avellino.it i ba.it i balsan.it i bari.it i barletta-trani-andria.it i barlettatraniandria.it i belluno.it i benevento.it i bergamo.it i bg.it i bi.it i biella.it i bl.it i bn.it i bo.it i bologna.it i bolzano.it i bozen.it i br.it i brescia.it i brindisi.it i bs.it i bt.it i bz.it i ca.it i cagliari.it i caltanissetta.it i campidano-medio.it i campidanomedio.it i campobasso.it i carbonia-iglesias.it i carboniaiglesias.it i carrara-massa.it i carraramassa.it i caserta.it i catania.it i catanzaro.it i cb.it i ce.it i cesena-forli.it i cesenaforli.it i ch.it i chieti.it i ci.it i cl.it i cn.it i co.it i como.it i cosenza.it i cr.it i cremona.it i crotone.it i cs.it i ct.it i cuneo.it i cz.it i dell-ogliastra.it i dellogliastra.it i en.it i enna.it i fc.it i fe.it i fermo.it i ferrara.it i fg.it i fi.it i firenze.it i florence.it i fm.it i foggia.it i forli-cesena.it i forlicesena.it i fr.it i frosinone.it i ge.it i genoa.it i genova.it i go.it i gorizia.it i gr.it i grosseto.it i iglesias-carbonia.it i iglesiascarbonia.it i im.it i imperia.it i is.it i isernia.it i kr.it i la-spezia.it i laquila.it i laspezia.it i latina.it i lc.it i le.it i lecce.it i lecco.it i li.it i livorno.it i lo.it i lodi.it i lt.it i lu.it i lucca.it i macerata.it i mantova.it i massa-carrara.it i massacarrara.it i matera.it i mb.it i mc.it i me.it i medio-campidano.it i mediocampidano.it i messina.it i mi.it i milan.it i milano.it i mn.it i mo.it i modena.it i monza-brianza.it i monza-e-della-brianza.it i monza.it i monzabrianza.it i monzaebrianza.it i monzaedellabrianza.it i ms.it i mt.it i na.it i naples.it i napoli.it i no.it i novara.it i nu.it i nuoro.it i og.it i ogliastra.it i olbia-tempio.it i olbiatempio.it i or.it i oristano.it i ot.it i pa.it i padova.it i padua.it i palermo.it i parma.it i pavia.it i pc.it i pd.it i pe.it i perugia.it i pesaro-urbino.it i pesarourbino.it i pescara.it i pg.it i pi.it i piacenza.it i pisa.it i pistoia.it i pn.it i po.it i pordenone.it i potenza.it i pr.it i prato.it i pt.it i pu.it i pv.it i pz.it i ra.it i ragusa.it i ravenna.it i rc.it i re.it i reggio-calabria.it i reggio-emilia.it i reggiocalabria.it i reggioemilia.it i rg.it i ri.it i rieti.it i rimini.it i rm.it i rn.it i ro.it i roma.it i rome.it i rovigo.it i sa.it i salerno.it i sassari.it i savona.it i si.it i siena.it i siracusa.it i so.it i sondrio.it i sp.it i sr.it i ss.it i suedtirol.it i sv.it i ta.it i taranto.it i te.it i tempio-olbia.it i tempioolbia.it i teramo.it i terni.it i tn.it i to.it i torino.it i tp.it i tr.it i trani-andria-barletta.it i trani-barletta-andria.it i traniandriabarletta.it i tranibarlettaandria.it i trapani.it i trentino.it i trento.it i treviso.it i trieste.it i ts.it i turin.it i tv.it i ud.it i udine.it i urbino-pesaro.it i urbinopesaro.it i va.it i varese.it i vb.it i vc.it i ve.it i venezia.it i venice.it i verbania.it i vercelli.it i verona.it i vi.it i vibo-valentia.it i vibovalentia.it i vicenza.it i viterbo.it i vr.it i vs.it i vt.it i vv.it i je i co.je i net.je i org.je i jm w jo i com.jo i org.jo i net.jo i edu.jo i sch.jo i gov.jo i mil.jo i name.jo i jobs i jp i ac.jp i ad.jp i co.jp i ed.jp i go.jp i gr.jp i lg.jp i ne.jp i or.jp i aichi.jp i akita.jp i aomori.jp i chiba.jp i ehime.jp i fukui.jp i fukuoka.jp i fukushima.jp i gifu.jp i gunma.jp i hiroshima.jp i hokkaido.jp i hyogo.jp i ibaraki.jp i ishikawa.jp i iwate.jp i kagawa.jp i kagoshima.jp i kanagawa.jp i kochi.jp i kumamoto.jp i kyoto.jp i mie.jp i miyagi.jp i miyazaki.jp i nagano.jp i nagasaki.jp i nara.jp i niigata.jp i oita.jp i okayama.jp i okinawa.jp i osaka.jp i saga.jp i saitama.jp i shiga.jp i shimane.jp i shizuoka.jp i tochigi.jp i tokushima.jp i tokyo.jp i tottori.jp i toyama.jp i wakayama.jp i yamagata.jp i yamaguchi.jp i yamanashi.jp i kawasaki.jp w kitakyushu.jp w kobe.jp w nagoya.jp w sapporo.jp w sendai.jp w yokohama.jp w city.kawasaki.jp e city.kitakyushu.jp e city.kobe.jp e city.nagoya.jp e city.sapporo.jp e city.sendai.jp e city.yokohama.jp e aisai.aichi.jp i ama.aichi.jp i anjo.aichi.jp i asuke.aichi.jp i chiryu.aichi.jp i chita.aichi.jp i fuso.aichi.jp i gamagori.aichi.jp i handa.aichi.jp i hazu.aichi.jp i hekinan.aichi.jp i higashiura.aichi.jp i ichinomiya.aichi.jp i inazawa.aichi.jp i inuyama.aichi.jp i isshiki.aichi.jp i iwakura.aichi.jp i kanie.aichi.jp i kariya.aichi.jp i kasugai.aichi.jp i kira.aichi.jp i kiyosu.aichi.jp i komaki.aichi.jp i konan.aichi.jp i kota.aichi.jp i mihama.aichi.jp i miyoshi.aichi.jp i nishio.aichi.jp i nisshin.aichi.jp i obu.aichi.jp i oguchi.aichi.jp i oharu.aichi.jp i okazaki.aichi.jp i owariasahi.aichi.jp i seto.aichi.jp i shikatsu.aichi.jp i shinshiro.aichi.jp i shitara.aichi.jp i tahara.aichi.jp i takahama.aichi.jp i tobishima.aichi.jp i toei.aichi.jp i togo.aichi.jp i tokai.aichi.jp i tokoname.aichi.jp i toyoake.aichi.jp i toyohashi.aichi.jp i toyokawa.aichi.jp i toyone.aichi.jp i toyota.aichi.jp i tsushima.aichi.jp i yatomi.aichi.jp i akita.akita.jp i daisen.akita.jp i fujisato.akita.jp i gojome.akita.jp i hachirogata.akita.jp i happou.akita.jp i higashinaruse.akita.jp i honjo.akita.jp i honjyo.akita.jp i ikawa.akita.jp i kamikoani.akita.jp i kamioka.akita.jp i katagami.akita.jp i kazuno.akita.jp i kitaakita.akita.jp i kosaka.akita.jp i kyowa.akita.jp i misato.akita.jp i mitane.akita.jp i moriyoshi.akita.jp i nikaho.akita.jp i noshiro.akita.jp i odate.akita.jp i oga.akita.jp i ogata.akita.jp i semboku.akita.jp i yokote.akita.jp i yurihonjo.akita.jp i aomori.aomori.jp i gonohe.aomori.jp i hachinohe.aomori.jp i hashikami.aomori.jp i hiranai.aomori.jp i hirosaki.aomori.jp i itayanagi.aomori.jp i kuroishi.aomori.jp i misawa.aomori.jp i mutsu.aomori.jp i nakadomari.aomori.jp i noheji.aomori.jp i oirase.aomori.jp i owani.aomori.jp i rokunohe.aomori.jp i sannohe.aomori.jp i shichinohe.aomori.jp i shingo.aomori.jp i takko.aomori.jp i towada.aomori.jp i tsugaru.aomori.jp i tsuruta.aomori.jp i abiko.chiba.jp i asahi.chiba.jp i chonan.chiba.jp i chosei.chiba.jp i choshi.chiba.jp i chuo.chiba.jp i funabashi.chiba.jp i futtsu.chiba.jp i hanamigawa.chiba.jp i ichihara.chiba.jp i ichikawa.chiba.jp i ichinomiya.chiba.jp i inzai.chiba.jp i isumi.chiba.jp i kamagaya.chiba.jp i kamogawa.chiba.jp i kashiwa.chiba.jp i katori.chiba.jp i katsuura.chiba.jp i kimitsu.chiba.jp i kisarazu.chiba.jp i kozaki.chiba.jp i kujukuri.chiba.jp i kyonan.chiba.jp i matsudo.chiba.jp i midori.chiba.jp i mihama.chiba.jp i minamiboso.chiba.jp i mobara.chiba.jp i mutsuzawa.chiba.jp i nagara.chiba.jp i nagareyama.chiba.jp i narashino.chiba.jp i narita.chiba.jp i noda.chiba.jp i oamishirasato.chiba.jp i omigawa.chiba.jp i onjuku.chiba.jp i otaki.chiba.jp i sakae.chiba.jp i sakura.chiba.jp i shimofusa.chiba.jp i shirako.chiba.jp i shiroi.chiba.jp i shisui.chiba.jp i sodegaura.chiba.jp i sosa.chiba.jp i tako.chiba.jp i tateyama.chiba.jp i togane.chiba.jp i tohnosho.chiba.jp i tomisato.chiba.jp i urayasu.chiba.jp i yachimata.chiba.jp i yachiyo.chiba.jp i yokaichiba.chiba.jp i yokoshibahikari.chiba.jp i yotsukaido.chiba.jp i ainan.ehime.jp i honai.ehime.jp i ikata.ehime.jp i imabari.ehime.jp i iyo.ehime.jp i kamijima.ehime.jp i kihoku.ehime.jp i kumakogen.ehime.jp i masaki.ehime.jp i matsuno.ehime.jp i matsuyama.ehime.jp i namikata.ehime.jp i niihama.ehime.jp i ozu.ehime.jp i saijo.ehime.jp i seiyo.ehime.jp i shikokuchuo.ehime.jp i tobe.ehime.jp i toon.ehime.jp i uchiko.ehime.jp i uwajima.ehime.jp i yawatahama.ehime.jp i echizen.fukui.jp i eiheiji.fukui.jp i fukui.fukui.jp i ikeda.fukui.jp i katsuyama.fukui.jp i mihama.fukui.jp i minamiechizen.fukui.jp i obama.fukui.jp i ohi.fukui.jp i ono.fukui.jp i sabae.fukui.jp i sakai.fukui.jp i takahama.fukui.jp i tsuruga.fukui.jp i wakasa.fukui.jp i ashiya.fukuoka.jp i buzen.fukuoka.jp i chikugo.fukuoka.jp i chikuho.fukuoka.jp i chikujo.fukuoka.jp i chikushino.fukuoka.jp i chikuzen.fukuoka.jp i chuo.fukuoka.jp i dazaifu.fukuoka.jp i fukuchi.fukuoka.jp i hakata.fukuoka.jp i higashi.fukuoka.jp i hirokawa.fukuoka.jp i hisayama.fukuoka.jp i iizuka.fukuoka.jp i inatsuki.fukuoka.jp i kaho.fukuoka.jp i kasuga.fukuoka.jp i kasuya.fukuoka.jp i kawara.fukuoka.jp i keisen.fukuoka.jp i koga.fukuoka.jp i kurate.fukuoka.jp i kurogi.fukuoka.jp i kurume.fukuoka.jp i minami.fukuoka.jp i miyako.fukuoka.jp i miyama.fukuoka.jp i miyawaka.fukuoka.jp i mizumaki.fukuoka.jp i munakata.fukuoka.jp i nakagawa.fukuoka.jp i nakama.fukuoka.jp i nishi.fukuoka.jp i nogata.fukuoka.jp i ogori.fukuoka.jp i okagaki.fukuoka.jp i okawa.fukuoka.jp i oki.fukuoka.jp i omuta.fukuoka.jp i onga.fukuoka.jp i onojo.fukuoka.jp i oto.fukuoka.jp i saigawa.fukuoka.jp i sasaguri.fukuoka.jp i shingu.fukuoka.jp i shinyoshitomi.fukuoka.jp i shonai.fukuoka.jp i soeda.fukuoka.jp i sue.fukuoka.jp i tachiarai.fukuoka.jp i tagawa.fukuoka.jp i takata.fukuoka.jp i toho.fukuoka.jp i toyotsu.fukuoka.jp i tsuiki.fukuoka.jp i ukiha.fukuoka.jp i umi.fukuoka.jp i usui.fukuoka.jp i yamada.fukuoka.jp i yame.fukuoka.jp i yanagawa.fukuoka.jp i yukuhashi.fukuoka.jp i aizubange.fukushima.jp i aizumisato.fukushima.jp i aizuwakamatsu.fukushima.jp i asakawa.fukushima.jp i bandai.fukushima.jp i date.fukushima.jp i fukushima.fukushima.jp i furudono.fukushima.jp i futaba.fukushima.jp i hanawa.fukushima.jp i higashi.fukushima.jp i hirata.fukushima.jp i hirono.fukushima.jp i iitate.fukushima.jp i inawashiro.fukushima.jp i ishikawa.fukushima.jp i iwaki.fukushima.jp i izumizaki.fukushima.jp i kagamiishi.fukushima.jp i kaneyama.fukushima.jp i kawamata.fukushima.jp i kitakata.fukushima.jp i kitashiobara.fukushima.jp i koori.fukushima.jp i koriyama.fukushima.jp i kunimi.fukushima.jp i miharu.fukushima.jp i mishima.fukushima.jp i namie.fukushima.jp i nango.fukushima.jp i nishiaizu.fukushima.jp i nishigo.fukushima.jp i okuma.fukushima.jp i omotego.fukushima.jp i ono.fukushima.jp i otama.fukushima.jp i samegawa.fukushima.jp i shimogo.fukushima.jp i shirakawa.fukushima.jp i showa.fukushima.jp i soma.fukushima.jp i sukagawa.fukushima.jp i taishin.fukushima.jp i tamakawa.fukushima.jp i tanagura.fukushima.jp i tenei.fukushima.jp i yabuki.fukushima.jp i yamato.fukushima.jp i yamatsuri.fukushima.jp i yanaizu.fukushima.jp i yugawa.fukushima.jp i anpachi.gifu.jp i ena.gifu.jp i gifu.gifu.jp i ginan.gifu.jp i godo.gifu.jp i gujo.gifu.jp i hashima.gifu.jp i hichiso.gifu.jp i hida.gifu.jp i higashishirakawa.gifu.jp i ibigawa.gifu.jp i ikeda.gifu.jp i kakamigahara.gifu.jp i kani.gifu.jp i kasahara.gifu.jp i kasamatsu.gifu.jp i kawaue.gifu.jp i kitagata.gifu.jp i mino.gifu.jp i minokamo.gifu.jp i mitake.gifu.jp i mizunami.gifu.jp i motosu.gifu.jp i nakatsugawa.gifu.jp i ogaki.gifu.jp i sakahogi.gifu.jp i seki.gifu.jp i sekigahara.gifu.jp i shirakawa.gifu.jp i tajimi.gifu.jp i takayama.gifu.jp i tarui.gifu.jp i toki.gifu.jp i tomika.gifu.jp i wanouchi.gifu.jp i yamagata.gifu.jp i yaotsu.gifu.jp i yoro.gifu.jp i annaka.gunma.jp i chiyoda.gunma.jp i fujioka.gunma.jp i higashiagatsuma.gunma.jp i isesaki.gunma.jp i itakura.gunma.jp i kanna.gunma.jp i kanra.gunma.jp i katashina.gunma.jp i kawaba.gunma.jp i kiryu.gunma.jp i kusatsu.gunma.jp i maebashi.gunma.jp i meiwa.gunma.jp i midori.gunma.jp i minakami.gunma.jp i naganohara.gunma.jp i nakanojo.gunma.jp i nanmoku.gunma.jp i numata.gunma.jp i oizumi.gunma.jp i ora.gunma.jp i ota.gunma.jp i shibukawa.gunma.jp i shimonita.gunma.jp i shinto.gunma.jp i showa.gunma.jp i takasaki.gunma.jp i takayama.gunma.jp i tamamura.gunma.jp i tatebayashi.gunma.jp i tomioka.gunma.jp i tsukiyono.gunma.jp i tsumagoi.gunma.jp i ueno.gunma.jp i yoshioka.gunma.jp i asaminami.hiroshima.jp i daiwa.hiroshima.jp i etajima.hiroshima.jp i fuchu.hiroshima.jp i fukuyama.hiroshima.jp i hatsukaichi.hiroshima.jp i higashihiroshima.hiroshima.jp i hongo.hiroshima.jp i jinsekikogen.hiroshima.jp i kaita.hiroshima.jp i kui.hiroshima.jp i kumano.hiroshima.jp i kure.hiroshima.jp i mihara.hiroshima.jp i miyoshi.hiroshima.jp i naka.hiroshima.jp i onomichi.hiroshima.jp i osakikamijima.hiroshima.jp i otake.hiroshima.jp i saka.hiroshima.jp i sera.hiroshima.jp i seranishi.hiroshima.jp i shinichi.hiroshima.jp i shobara.hiroshima.jp i takehara.hiroshima.jp i abashiri.hokkaido.jp i abira.hokkaido.jp i aibetsu.hokkaido.jp i akabira.hokkaido.jp i akkeshi.hokkaido.jp i asahikawa.hokkaido.jp i ashibetsu.hokkaido.jp i ashoro.hokkaido.jp i assabu.hokkaido.jp i atsuma.hokkaido.jp i bibai.hokkaido.jp i biei.hokkaido.jp i bifuka.hokkaido.jp i bihoro.hokkaido.jp i biratori.hokkaido.jp i chippubetsu.hokkaido.jp i chitose.hokkaido.jp i date.hokkaido.jp i ebetsu.hokkaido.jp i embetsu.hokkaido.jp i eniwa.hokkaido.jp i erimo.hokkaido.jp i esan.hokkaido.jp i esashi.hokkaido.jp i fukagawa.hokkaido.jp i fukushima.hokkaido.jp i furano.hokkaido.jp i furubira.hokkaido.jp i haboro.hokkaido.jp i hakodate.hokkaido.jp i hamatonbetsu.hokkaido.jp i hidaka.hokkaido.jp i higashikagura.hokkaido.jp i higashikawa.hokkaido.jp i hiroo.hokkaido.jp i hokuryu.hokkaido.jp i hokuto.hokkaido.jp i honbetsu.hokkaido.jp i horokanai.hokkaido.jp i horonobe.hokkaido.jp i ikeda.hokkaido.jp i imakane.hokkaido.jp i ishikari.hokkaido.jp i iwamizawa.hokkaido.jp i iwanai.hokkaido.jp i kamifurano.hokkaido.jp i kamikawa.hokkaido.jp i kamishihoro.hokkaido.jp i kamisunagawa.hokkaido.jp i kamoenai.hokkaido.jp i kayabe.hokkaido.jp i kembuchi.hokkaido.jp i kikonai.hokkaido.jp i kimobetsu.hokkaido.jp i kitahiroshima.hokkaido.jp i kitami.hokkaido.jp i kiyosato.hokkaido.jp i koshimizu.hokkaido.jp i kunneppu.hokkaido.jp i kuriyama.hokkaido.jp i kuromatsunai.hokkaido.jp i kushiro.hokkaido.jp i kutchan.hokkaido.jp i kyowa.hokkaido.jp i mashike.hokkaido.jp i matsumae.hokkaido.jp i mikasa.hokkaido.jp i minamifurano.hokkaido.jp i mombetsu.hokkaido.jp i moseushi.hokkaido.jp i mukawa.hokkaido.jp i muroran.hokkaido.jp i naie.hokkaido.jp i nakagawa.hokkaido.jp i nakasatsunai.hokkaido.jp i nakatombetsu.hokkaido.jp i nanae.hokkaido.jp i nanporo.hokkaido.jp i nayoro.hokkaido.jp i nemuro.hokkaido.jp i niikappu.hokkaido.jp i niki.hokkaido.jp i nishiokoppe.hokkaido.jp i noboribetsu.hokkaido.jp i numata.hokkaido.jp i obihiro.hokkaido.jp i obira.hokkaido.jp i oketo.hokkaido.jp i okoppe.hokkaido.jp i otaru.hokkaido.jp i otobe.hokkaido.jp i otofuke.hokkaido.jp i otoineppu.hokkaido.jp i oumu.hokkaido.jp i ozora.hokkaido.jp i pippu.hokkaido.jp i rankoshi.hokkaido.jp i rebun.hokkaido.jp i rikubetsu.hokkaido.jp i rishiri.hokkaido.jp i rishirifuji.hokkaido.jp i saroma.hokkaido.jp i sarufutsu.hokkaido.jp i shakotan.hokkaido.jp i shari.hokkaido.jp i shibecha.hokkaido.jp i shibetsu.hokkaido.jp i shikabe.hokkaido.jp i shikaoi.hokkaido.jp i shimamaki.hokkaido.jp i shimizu.hokkaido.jp i shimokawa.hokkaido.jp i shinshinotsu.hokkaido.jp i shintoku.hokkaido.jp i shiranuka.hokkaido.jp i shiraoi.hokkaido.jp i shiriuchi.hokkaido.jp i sobetsu.hokkaido.jp i sunagawa.hokkaido.jp i taiki.hokkaido.jp i takasu.hokkaido.jp i takikawa.hokkaido.jp i takinoue.hokkaido.jp i teshikaga.hokkaido.jp i tobetsu.hokkaido.jp i tohma.hokkaido.jp i tomakomai.hokkaido.jp i tomari.hokkaido.jp i toya.hokkaido.jp i toyako.hokkaido.jp i toyotomi.hokkaido.jp i toyoura.hokkaido.jp i tsubetsu.hokkaido.jp i tsukigata.hokkaido.jp i urakawa.hokkaido.jp i urausu.hokkaido.jp i uryu.hokkaido.jp i utashinai.hokkaido.jp i wakkanai.hokkaido.jp i wassamu.hokkaido.jp i yakumo.hokkaido.jp i yoichi.hokkaido.jp i aioi.hyogo.jp i akashi.hyogo.jp i ako.hyogo.jp i amagasaki.hyogo.jp i aogaki.hyogo.jp i asago.hyogo.jp i ashiya.hyogo.jp i awaji.hyogo.jp i fukusaki.hyogo.jp i goshiki.hyogo.jp i harima.hyogo.jp i himeji.hyogo.jp i ichikawa.hyogo.jp i inagawa.hyogo.jp i itami.hyogo.jp i kakogawa.hyogo.jp i kamigori.hyogo.jp i kamikawa.hyogo.jp i kasai.hyogo.jp i kasuga.hyogo.jp i kawanishi.hyogo.jp i miki.hyogo.jp i minamiawaji.hyogo.jp i nishinomiya.hyogo.jp i nishiwaki.hyogo.jp i ono.hyogo.jp i sanda.hyogo.jp i sannan.hyogo.jp i sasayama.hyogo.jp i sayo.hyogo.jp i shingu.hyogo.jp i shinonsen.hyogo.jp i shiso.hyogo.jp i sumoto.hyogo.jp i taishi.hyogo.jp i taka.hyogo.jp i takarazuka.hyogo.jp i takasago.hyogo.jp i takino.hyogo.jp i tamba.hyogo.jp i tatsuno.hyogo.jp i toyooka.hyogo.jp i yabu.hyogo.jp i yashiro.hyogo.jp i yoka.hyogo.jp i yokawa.hyogo.jp i ami.ibaraki.jp i asahi.ibaraki.jp i bando.ibaraki.jp i chikusei.ibaraki.jp i daigo.ibaraki.jp i fujishiro.ibaraki.jp i hitachi.ibaraki.jp i hitachinaka.ibaraki.jp i hitachiomiya.ibaraki.jp i hitachiota.ibaraki.jp i ibaraki.ibaraki.jp i ina.ibaraki.jp i inashiki.ibaraki.jp i itako.ibaraki.jp i iwama.ibaraki.jp i joso.ibaraki.jp i kamisu.ibaraki.jp i kasama.ibaraki.jp i kashima.ibaraki.jp i kasumigaura.ibaraki.jp i koga.ibaraki.jp i miho.ibaraki.jp i mito.ibaraki.jp i moriya.ibaraki.jp i naka.ibaraki.jp i namegata.ibaraki.jp i oarai.ibaraki.jp i ogawa.ibaraki.jp i omitama.ibaraki.jp i ryugasaki.ibaraki.jp i sakai.ibaraki.jp i sakuragawa.ibaraki.jp i shimodate.ibaraki.jp i shimotsuma.ibaraki.jp i shirosato.ibaraki.jp i sowa.ibaraki.jp i suifu.ibaraki.jp i takahagi.ibaraki.jp i tamatsukuri.ibaraki.jp i tokai.ibaraki.jp i tomobe.ibaraki.jp i tone.ibaraki.jp i toride.ibaraki.jp i tsuchiura.ibaraki.jp i tsukuba.ibaraki.jp i uchihara.ibaraki.jp i ushiku.ibaraki.jp i yachiyo.ibaraki.jp i yamagata.ibaraki.jp i yawara.ibaraki.jp i yuki.ibaraki.jp i anamizu.ishikawa.jp i hakui.ishikawa.jp i hakusan.ishikawa.jp i kaga.ishikawa.jp i kahoku.ishikawa.jp i kanazawa.ishikawa.jp i kawakita.ishikawa.jp i komatsu.ishikawa.jp i nakanoto.ishikawa.jp i nanao.ishikawa.jp i nomi.ishikawa.jp i nonoichi.ishikawa.jp i noto.ishikawa.jp i shika.ishikawa.jp i suzu.ishikawa.jp i tsubata.ishikawa.jp i tsurugi.ishikawa.jp i uchinada.ishikawa.jp i wajima.ishikawa.jp i fudai.iwate.jp i fujisawa.iwate.jp i hanamaki.iwate.jp i hiraizumi.iwate.jp i hirono.iwate.jp i ichinohe.iwate.jp i ichinoseki.iwate.jp i iwaizumi.iwate.jp i iwate.iwate.jp i joboji.iwate.jp i kamaishi.iwate.jp i kanegasaki.iwate.jp i karumai.iwate.jp i kawai.iwate.jp i kitakami.iwate.jp i kuji.iwate.jp i kunohe.iwate.jp i kuzumaki.iwate.jp i miyako.iwate.jp i mizusawa.iwate.jp i morioka.iwate.jp i ninohe.iwate.jp i noda.iwate.jp i ofunato.iwate.jp i oshu.iwate.jp i otsuchi.iwate.jp i rikuzentakata.iwate.jp i shiwa.iwate.jp i shizukuishi.iwate.jp i sumita.iwate.jp i tanohata.iwate.jp i tono.iwate.jp i yahaba.iwate.jp i yamada.iwate.jp i ayagawa.kagawa.jp i higashikagawa.kagawa.jp i kanonji.kagawa.jp i kotohira.kagawa.jp i manno.kagawa.jp i marugame.kagawa.jp i mitoyo.kagawa.jp i naoshima.kagawa.jp i sanuki.kagawa.jp i tadotsu.kagawa.jp i takamatsu.kagawa.jp i tonosho.kagawa.jp i uchinomi.kagawa.jp i utazu.kagawa.jp i zentsuji.kagawa.jp i akune.kagoshima.jp i amami.kagoshima.jp i hioki.kagoshima.jp i isa.kagoshima.jp i isen.kagoshima.jp i izumi.kagoshima.jp i kagoshima.kagoshima.jp i kanoya.kagoshima.jp i kawanabe.kagoshima.jp i kinko.kagoshima.jp i kouyama.kagoshima.jp i makurazaki.kagoshima.jp i matsumoto.kagoshima.jp i minamitane.kagoshima.jp i nakatane.kagoshima.jp i nishinoomote.kagoshima.jp i satsumasendai.kagoshima.jp i soo.kagoshima.jp i tarumizu.kagoshima.jp i yusui.kagoshima.jp i aikawa.kanagawa.jp i atsugi.kanagawa.jp i ayase.kanagawa.jp i chigasaki.kanagawa.jp i ebina.kanagawa.jp i fujisawa.kanagawa.jp i hadano.kanagawa.jp i hakone.kanagawa.jp i hiratsuka.kanagawa.jp i isehara.kanagawa.jp i kaisei.kanagawa.jp i kamakura.kanagawa.jp i kiyokawa.kanagawa.jp i matsuda.kanagawa.jp i minamiashigara.kanagawa.jp i miura.kanagawa.jp i nakai.kanagawa.jp i ninomiya.kanagawa.jp i odawara.kanagawa.jp i oi.kanagawa.jp i oiso.kanagawa.jp i sagamihara.kanagawa.jp i samukawa.kanagawa.jp i tsukui.kanagawa.jp i yamakita.kanagawa.jp i yamato.kanagawa.jp i yokosuka.kanagawa.jp i yugawara.kanagawa.jp i zama.kanagawa.jp i zushi.kanagawa.jp i aki.kochi.jp i geisei.kochi.jp i hidaka.kochi.jp i higashitsuno.kochi.jp i ino.kochi.jp i kagami.kochi.jp i kami.kochi.jp i kitagawa.kochi.jp i kochi.kochi.jp i mihara.kochi.jp i motoyama.kochi.jp i muroto.kochi.jp i nahari.kochi.jp i nakamura.kochi.jp i nankoku.kochi.jp i nishitosa.kochi.jp i niyodogawa.kochi.jp i ochi.kochi.jp i okawa.kochi.jp i otoyo.kochi.jp i otsuki.kochi.jp i sakawa.kochi.jp i sukumo.kochi.jp i susaki.kochi.jp i tosa.kochi.jp i tosashimizu.kochi.jp i toyo.kochi.jp i tsuno.kochi.jp i umaji.kochi.jp i yasuda.kochi.jp i yusuhara.kochi.jp i amakusa.kumamoto.jp i arao.kumamoto.jp i aso.kumamoto.jp i choyo.kumamoto.jp i gyokuto.kumamoto.jp i hitoyoshi.kumamoto.jp i kamiamakusa.kumamoto.jp i kashima.kumamoto.jp i kikuchi.kumamoto.jp i kosa.kumamoto.jp i kumamoto.kumamoto.jp i mashiki.kumamoto.jp i mifune.kumamoto.jp i minamata.kumamoto.jp i minamioguni.kumamoto.jp i nagasu.kumamoto.jp i nishihara.kumamoto.jp i oguni.kumamoto.jp i ozu.kumamoto.jp i sumoto.kumamoto.jp i takamori.kumamoto.jp i uki.kumamoto.jp i uto.kumamoto.jp i yamaga.kumamoto.jp i yamato.kumamoto.jp i yatsushiro.kumamoto.jp i ayabe.kyoto.jp i fukuchiyama.kyoto.jp i higashiyama.kyoto.jp i ide.kyoto.jp i ine.kyoto.jp i joyo.kyoto.jp i kameoka.kyoto.jp i kamo.kyoto.jp i kita.kyoto.jp i kizu.kyoto.jp i kumiyama.kyoto.jp i kyotamba.kyoto.jp i kyotanabe.kyoto.jp i kyotango.kyoto.jp i maizuru.kyoto.jp i minami.kyoto.jp i minamiyamashiro.kyoto.jp i miyazu.kyoto.jp i muko.kyoto.jp i nagaokakyo.kyoto.jp i nakagyo.kyoto.jp i nantan.kyoto.jp i oyamazaki.kyoto.jp i sakyo.kyoto.jp i seika.kyoto.jp i tanabe.kyoto.jp i uji.kyoto.jp i ujitawara.kyoto.jp i wazuka.kyoto.jp i yamashina.kyoto.jp i yawata.kyoto.jp i asahi.mie.jp i inabe.mie.jp i ise.mie.jp i kameyama.mie.jp i kawagoe.mie.jp i kiho.mie.jp i kisosaki.mie.jp i kiwa.mie.jp i komono.mie.jp i kumano.mie.jp i kuwana.mie.jp i matsusaka.mie.jp i meiwa.mie.jp i mihama.mie.jp i minamiise.mie.jp i misugi.mie.jp i miyama.mie.jp i nabari.mie.jp i shima.mie.jp i suzuka.mie.jp i tado.mie.jp i taiki.mie.jp i taki.mie.jp i tamaki.mie.jp i toba.mie.jp i tsu.mie.jp i udono.mie.jp i ureshino.mie.jp i watarai.mie.jp i yokkaichi.mie.jp i furukawa.miyagi.jp i higashimatsushima.miyagi.jp i ishinomaki.miyagi.jp i iwanuma.miyagi.jp i kakuda.miyagi.jp i kami.miyagi.jp i kawasaki.miyagi.jp i kesennuma.miyagi.jp i marumori.miyagi.jp i matsushima.miyagi.jp i minamisanriku.miyagi.jp i misato.miyagi.jp i murata.miyagi.jp i natori.miyagi.jp i ogawara.miyagi.jp i ohira.miyagi.jp i onagawa.miyagi.jp i osaki.miyagi.jp i rifu.miyagi.jp i semine.miyagi.jp i shibata.miyagi.jp i shichikashuku.miyagi.jp i shikama.miyagi.jp i shiogama.miyagi.jp i shiroishi.miyagi.jp i tagajo.miyagi.jp i taiwa.miyagi.jp i tome.miyagi.jp i tomiya.miyagi.jp i wakuya.miyagi.jp i watari.miyagi.jp i yamamoto.miyagi.jp i zao.miyagi.jp i aya.miyazaki.jp i ebino.miyazaki.jp i gokase.miyazaki.jp i hyuga.miyazaki.jp i kadogawa.miyazaki.jp i kawaminami.miyazaki.jp i kijo.miyazaki.jp i kitagawa.miyazaki.jp i kitakata.miyazaki.jp i kitaura.miyazaki.jp i kobayashi.miyazaki.jp i kunitomi.miyazaki.jp i kushima.miyazaki.jp i mimata.miyazaki.jp i miyakonojo.miyazaki.jp i miyazaki.miyazaki.jp i morotsuka.miyazaki.jp i nichinan.miyazaki.jp i nishimera.miyazaki.jp i nobeoka.miyazaki.jp i saito.miyazaki.jp i shiiba.miyazaki.jp i shintomi.miyazaki.jp i takaharu.miyazaki.jp i takanabe.miyazaki.jp i takazaki.miyazaki.jp i tsuno.miyazaki.jp i achi.nagano.jp i agematsu.nagano.jp i anan.nagano.jp i aoki.nagano.jp i asahi.nagano.jp i azumino.nagano.jp i chikuhoku.nagano.jp i chikuma.nagano.jp i chino.nagano.jp i fujimi.nagano.jp i hakuba.nagano.jp i hara.nagano.jp i hiraya.nagano.jp i iida.nagano.jp i iijima.nagano.jp i iiyama.nagano.jp i iizuna.nagano.jp i ikeda.nagano.jp i ikusaka.nagano.jp i ina.nagano.jp i karuizawa.nagano.jp i kawakami.nagano.jp i kiso.nagano.jp i kisofukushima.nagano.jp i kitaaiki.nagano.jp i komagane.nagano.jp i komoro.nagano.jp i matsukawa.nagano.jp i matsumoto.nagano.jp i miasa.nagano.jp i minamiaiki.nagano.jp i minamimaki.nagano.jp i minamiminowa.nagano.jp i minowa.nagano.jp i miyada.nagano.jp i miyota.nagano.jp i mochizuki.nagano.jp i nagano.nagano.jp i nagawa.nagano.jp i nagiso.nagano.jp i nakagawa.nagano.jp i nakano.nagano.jp i nozawaonsen.nagano.jp i obuse.nagano.jp i ogawa.nagano.jp i okaya.nagano.jp i omachi.nagano.jp i omi.nagano.jp i ookuwa.nagano.jp i ooshika.nagano.jp i otaki.nagano.jp i otari.nagano.jp i sakae.nagano.jp i sakaki.nagano.jp i saku.nagano.jp i sakuho.nagano.jp i shimosuwa.nagano.jp i shinanomachi.nagano.jp i shiojiri.nagano.jp i suwa.nagano.jp i suzaka.nagano.jp i takagi.nagano.jp i takamori.nagano.jp i takayama.nagano.jp i tateshina.nagano.jp i tatsuno.nagano.jp i togakushi.nagano.jp i togura.nagano.jp i tomi.nagano.jp i ueda.nagano.jp i wada.nagano.jp i yamagata.nagano.jp i yamanouchi.nagano.jp i yasaka.nagano.jp i yasuoka.nagano.jp i chijiwa.nagasaki.jp i futsu.nagasaki.jp i goto.nagasaki.jp i hasami.nagasaki.jp i hirado.nagasaki.jp i iki.nagasaki.jp i isahaya.nagasaki.jp i kawatana.nagasaki.jp i kuchinotsu.nagasaki.jp i matsuura.nagasaki.jp i nagasaki.nagasaki.jp i obama.nagasaki.jp i omura.nagasaki.jp i oseto.nagasaki.jp i saikai.nagasaki.jp i sasebo.nagasaki.jp i seihi.nagasaki.jp i shimabara.nagasaki.jp i shinkamigoto.nagasaki.jp i togitsu.nagasaki.jp i tsushima.nagasaki.jp i unzen.nagasaki.jp i ando.nara.jp i gose.nara.jp i heguri.nara.jp i higashiyoshino.nara.jp i ikaruga.nara.jp i ikoma.nara.jp i kamikitayama.nara.jp i kanmaki.nara.jp i kashiba.nara.jp i kashihara.nara.jp i katsuragi.nara.jp i kawai.nara.jp i kawakami.nara.jp i kawanishi.nara.jp i koryo.nara.jp i kurotaki.nara.jp i mitsue.nara.jp i miyake.nara.jp i nara.nara.jp i nosegawa.nara.jp i oji.nara.jp i ouda.nara.jp i oyodo.nara.jp i sakurai.nara.jp i sango.nara.jp i shimoichi.nara.jp i shimokitayama.nara.jp i shinjo.nara.jp i soni.nara.jp i takatori.nara.jp i tawaramoto.nara.jp i tenkawa.nara.jp i tenri.nara.jp i uda.nara.jp i yamatokoriyama.nara.jp i yamatotakada.nara.jp i yamazoe.nara.jp i yoshino.nara.jp i aga.niigata.jp i agano.niigata.jp i gosen.niigata.jp i itoigawa.niigata.jp i izumozaki.niigata.jp i joetsu.niigata.jp i kamo.niigata.jp i kariwa.niigata.jp i kashiwazaki.niigata.jp i minamiuonuma.niigata.jp i mitsuke.niigata.jp i muika.niigata.jp i murakami.niigata.jp i myoko.niigata.jp i nagaoka.niigata.jp i niigata.niigata.jp i ojiya.niigata.jp i omi.niigata.jp i sado.niigata.jp i sanjo.niigata.jp i seiro.niigata.jp i seirou.niigata.jp i sekikawa.niigata.jp i shibata.niigata.jp i tagami.niigata.jp i tainai.niigata.jp i tochio.niigata.jp i tokamachi.niigata.jp i tsubame.niigata.jp i tsunan.niigata.jp i uonuma.niigata.jp i yahiko.niigata.jp i yoita.niigata.jp i yuzawa.niigata.jp i beppu.oita.jp i bungoono.oita.jp i bungotakada.oita.jp i hasama.oita.jp i hiji.oita.jp i himeshima.oita.jp i hita.oita.jp i kamitsue.oita.jp i kokonoe.oita.jp i kuju.oita.jp i kunisaki.oita.jp i kusu.oita.jp i oita.oita.jp i saiki.oita.jp i taketa.oita.jp i tsukumi.oita.jp i usa.oita.jp i usuki.oita.jp i yufu.oita.jp i akaiwa.okayama.jp i asakuchi.okayama.jp i bizen.okayama.jp i hayashima.okayama.jp i ibara.okayama.jp i kagamino.okayama.jp i kasaoka.okayama.jp i kibichuo.okayama.jp i kumenan.okayama.jp i kurashiki.okayama.jp i maniwa.okayama.jp i misaki.okayama.jp i nagi.okayama.jp i niimi.okayama.jp i nishiawakura.okayama.jp i okayama.okayama.jp i satosho.okayama.jp i setouchi.okayama.jp i shinjo.okayama.jp i shoo.okayama.jp i soja.okayama.jp i takahashi.okayama.jp i tamano.okayama.jp i tsuyama.okayama.jp i wake.okayama.jp i yakage.okayama.jp i aguni.okinawa.jp i ginowan.okinawa.jp i ginoza.okinawa.jp i gushikami.okinawa.jp i haebaru.okinawa.jp i higashi.okinawa.jp i hirara.okinawa.jp i iheya.okinawa.jp i ishigaki.okinawa.jp i ishikawa.okinawa.jp i itoman.okinawa.jp i izena.okinawa.jp i kadena.okinawa.jp i kin.okinawa.jp i kitadaito.okinawa.jp i kitanakagusuku.okinawa.jp i kumejima.okinawa.jp i kunigami.okinawa.jp i minamidaito.okinawa.jp i motobu.okinawa.jp i nago.okinawa.jp i naha.okinawa.jp i nakagusuku.okinawa.jp i nakijin.okinawa.jp i nanjo.okinawa.jp i nishihara.okinawa.jp i ogimi.okinawa.jp i okinawa.okinawa.jp i onna.okinawa.jp i shimoji.okinawa.jp i taketomi.okinawa.jp i tarama.okinawa.jp i tokashiki.okinawa.jp i tomigusuku.okinawa.jp i tonaki.okinawa.jp i urasoe.okinawa.jp i uruma.okinawa.jp i yaese.okinawa.jp i yomitan.okinawa.jp i yonabaru.okinawa.jp i yonaguni.okinawa.jp i zamami.okinawa.jp i abeno.osaka.jp i chihayaakasaka.osaka.jp i chuo.osaka.jp i daito.osaka.jp i fujiidera.osaka.jp i habikino.osaka.jp i hannan.osaka.jp i higashiosaka.osaka.jp i higashisumiyoshi.osaka.jp i higashiyodogawa.osaka.jp i hirakata.osaka.jp i ibaraki.osaka.jp i ikeda.osaka.jp i izumi.osaka.jp i izumiotsu.osaka.jp i izumisano.osaka.jp i kadoma.osaka.jp i kaizuka.osaka.jp i kanan.osaka.jp i kashiwara.osaka.jp i katano.osaka.jp i kawachinagano.osaka.jp i kishiwada.osaka.jp i kita.osaka.jp i kumatori.osaka.jp i matsubara.osaka.jp i minato.osaka.jp i minoh.osaka.jp i misaki.osaka.jp i moriguchi.osaka.jp i neyagawa.osaka.jp i nishi.osaka.jp i nose.osaka.jp i osakasayama.osaka.jp i sakai.osaka.jp i sayama.osaka.jp i sennan.osaka.jp i settsu.osaka.jp i shijonawate.osaka.jp i shimamoto.osaka.jp i suita.osaka.jp i tadaoka.osaka.jp i taishi.osaka.jp i tajiri.osaka.jp i takaishi.osaka.jp i takatsuki.osaka.jp i tondabayashi.osaka.jp i toyonaka.osaka.jp i toyono.osaka.jp i yao.osaka.jp i ariake.saga.jp i arita.saga.jp i fukudomi.saga.jp i genkai.saga.jp i hamatama.saga.jp i hizen.saga.jp i imari.saga.jp i kamimine.saga.jp i kanzaki.saga.jp i karatsu.saga.jp i kashima.saga.jp i kitagata.saga.jp i kitahata.saga.jp i kiyama.saga.jp i kouhoku.saga.jp i kyuragi.saga.jp i nishiarita.saga.jp i ogi.saga.jp i omachi.saga.jp i ouchi.saga.jp i saga.saga.jp i shiroishi.saga.jp i taku.saga.jp i tara.saga.jp i tosu.saga.jp i yoshinogari.saga.jp i arakawa.saitama.jp i asaka.saitama.jp i chichibu.saitama.jp i fujimi.saitama.jp i fujimino.saitama.jp i fukaya.saitama.jp i hanno.saitama.jp i hanyu.saitama.jp i hasuda.saitama.jp i hatogaya.saitama.jp i hatoyama.saitama.jp i hidaka.saitama.jp i higashichichibu.saitama.jp i higashimatsuyama.saitama.jp i honjo.saitama.jp i ina.saitama.jp i iruma.saitama.jp i iwatsuki.saitama.jp i kamiizumi.saitama.jp i kamikawa.saitama.jp i kamisato.saitama.jp i kasukabe.saitama.jp i kawagoe.saitama.jp i kawaguchi.saitama.jp i kawajima.saitama.jp i kazo.saitama.jp i kitamoto.saitama.jp i koshigaya.saitama.jp i kounosu.saitama.jp i kuki.saitama.jp i kumagaya.saitama.jp i matsubushi.saitama.jp i minano.saitama.jp i misato.saitama.jp i miyashiro.saitama.jp i miyoshi.saitama.jp i moroyama.saitama.jp i nagatoro.saitama.jp i namegawa.saitama.jp i niiza.saitama.jp i ogano.saitama.jp i ogawa.saitama.jp i ogose.saitama.jp i okegawa.saitama.jp i omiya.saitama.jp i otaki.saitama.jp i ranzan.saitama.jp i ryokami.saitama.jp i saitama.saitama.jp i sakado.saitama.jp i satte.saitama.jp i sayama.saitama.jp i shiki.saitama.jp i shiraoka.saitama.jp i soka.saitama.jp i sugito.saitama.jp i toda.saitama.jp i tokigawa.saitama.jp i tokorozawa.saitama.jp i tsurugashima.saitama.jp i urawa.saitama.jp i warabi.saitama.jp i yashio.saitama.jp i yokoze.saitama.jp i yono.saitama.jp i yorii.saitama.jp i yoshida.saitama.jp i yoshikawa.saitama.jp i yoshimi.saitama.jp i aisho.shiga.jp i gamo.shiga.jp i higashiomi.shiga.jp i hikone.shiga.jp i koka.shiga.jp i konan.shiga.jp i kosei.shiga.jp i koto.shiga.jp i kusatsu.shiga.jp i maibara.shiga.jp i moriyama.shiga.jp i nagahama.shiga.jp i nishiazai.shiga.jp i notogawa.shiga.jp i omihachiman.shiga.jp i otsu.shiga.jp i ritto.shiga.jp i ryuoh.shiga.jp i takashima.shiga.jp i takatsuki.shiga.jp i torahime.shiga.jp i toyosato.shiga.jp i yasu.shiga.jp i akagi.shimane.jp i ama.shimane.jp i gotsu.shimane.jp i hamada.shimane.jp i higashiizumo.shimane.jp i hikawa.shimane.jp i hikimi.shimane.jp i izumo.shimane.jp i kakinoki.shimane.jp i masuda.shimane.jp i matsue.shimane.jp i misato.shimane.jp i nishinoshima.shimane.jp i ohda.shimane.jp i okinoshima.shimane.jp i okuizumo.shimane.jp i shimane.shimane.jp i tamayu.shimane.jp i tsuwano.shimane.jp i unnan.shimane.jp i yakumo.shimane.jp i yasugi.shimane.jp i yatsuka.shimane.jp i arai.shizuoka.jp i atami.shizuoka.jp i fuji.shizuoka.jp i fujieda.shizuoka.jp i fujikawa.shizuoka.jp i fujinomiya.shizuoka.jp i fukuroi.shizuoka.jp i gotemba.shizuoka.jp i haibara.shizuoka.jp i hamamatsu.shizuoka.jp i higashiizu.shizuoka.jp i ito.shizuoka.jp i iwata.shizuoka.jp i izu.shizuoka.jp i izunokuni.shizuoka.jp i kakegawa.shizuoka.jp i kannami.shizuoka.jp i kawanehon.shizuoka.jp i kawazu.shizuoka.jp i kikugawa.shizuoka.jp i kosai.shizuoka.jp i makinohara.shizuoka.jp i matsuzaki.shizuoka.jp i minamiizu.shizuoka.jp i mishima.shizuoka.jp i morimachi.shizuoka.jp i nishiizu.shizuoka.jp i numazu.shizuoka.jp i omaezaki.shizuoka.jp i shimada.shizuoka.jp i shimizu.shizuoka.jp i shimoda.shizuoka.jp i shizuoka.shizuoka.jp i susono.shizuoka.jp i yaizu.shizuoka.jp i yoshida.shizuoka.jp i ashikaga.tochigi.jp i bato.tochigi.jp i haga.tochigi.jp i ichikai.tochigi.jp i iwafune.tochigi.jp i kaminokawa.tochigi.jp i kanuma.tochigi.jp i karasuyama.tochigi.jp i kuroiso.tochigi.jp i mashiko.tochigi.jp i mibu.tochigi.jp i moka.tochigi.jp i motegi.tochigi.jp i nasu.tochigi.jp i nasushiobara.tochigi.jp i nikko.tochigi.jp i nishikata.tochigi.jp i nogi.tochigi.jp i ohira.tochigi.jp i ohtawara.tochigi.jp i oyama.tochigi.jp i sakura.tochigi.jp i sano.tochigi.jp i shimotsuke.tochigi.jp i shioya.tochigi.jp i takanezawa.tochigi.jp i tochigi.tochigi.jp i tsuga.tochigi.jp i ujiie.tochigi.jp i utsunomiya.tochigi.jp i yaita.tochigi.jp i aizumi.tokushima.jp i anan.tokushima.jp i ichiba.tokushima.jp i itano.tokushima.jp i kainan.tokushima.jp i komatsushima.tokushima.jp i matsushige.tokushima.jp i mima.tokushima.jp i minami.tokushima.jp i miyoshi.tokushima.jp i mugi.tokushima.jp i nakagawa.tokushima.jp i naruto.tokushima.jp i sanagochi.tokushima.jp i shishikui.tokushima.jp i tokushima.tokushima.jp i wajiki.tokushima.jp i adachi.tokyo.jp i akiruno.tokyo.jp i akishima.tokyo.jp i aogashima.tokyo.jp i arakawa.tokyo.jp i bunkyo.tokyo.jp i chiyoda.tokyo.jp i chofu.tokyo.jp i chuo.tokyo.jp i edogawa.tokyo.jp i fuchu.tokyo.jp i fussa.tokyo.jp i hachijo.tokyo.jp i hachioji.tokyo.jp i hamura.tokyo.jp i higashikurume.tokyo.jp i higashimurayama.tokyo.jp i higashiyamato.tokyo.jp i hino.tokyo.jp i hinode.tokyo.jp i hinohara.tokyo.jp i inagi.tokyo.jp i itabashi.tokyo.jp i katsushika.tokyo.jp i kita.tokyo.jp i kiyose.tokyo.jp i kodaira.tokyo.jp i koganei.tokyo.jp i kokubunji.tokyo.jp i komae.tokyo.jp i koto.tokyo.jp i kouzushima.tokyo.jp i kunitachi.tokyo.jp i machida.tokyo.jp i meguro.tokyo.jp i minato.tokyo.jp i mitaka.tokyo.jp i mizuho.tokyo.jp i musashimurayama.tokyo.jp i musashino.tokyo.jp i nakano.tokyo.jp i nerima.tokyo.jp i ogasawara.tokyo.jp i okutama.tokyo.jp i ome.tokyo.jp i oshima.tokyo.jp i ota.tokyo.jp i setagaya.tokyo.jp i shibuya.tokyo.jp i shinagawa.tokyo.jp i shinjuku.tokyo.jp i suginami.tokyo.jp i sumida.tokyo.jp i tachikawa.tokyo.jp i taito.tokyo.jp i tama.tokyo.jp i toshima.tokyo.jp i chizu.tottori.jp i hino.tottori.jp i kawahara.tottori.jp i koge.tottori.jp i kotoura.tottori.jp i misasa.tottori.jp i nanbu.tottori.jp i nichinan.tottori.jp i sakaiminato.tottori.jp i tottori.tottori.jp i wakasa.tottori.jp i yazu.tottori.jp i yonago.tottori.jp i asahi.toyama.jp i fuchu.toyama.jp i fukumitsu.toyama.jp i funahashi.toyama.jp i himi.toyama.jp i imizu.toyama.jp i inami.toyama.jp i johana.toyama.jp i kamiichi.toyama.jp i kurobe.toyama.jp i nakaniikawa.toyama.jp i namerikawa.toyama.jp i nanto.toyama.jp i nyuzen.toyama.jp i oyabe.toyama.jp i taira.toyama.jp i takaoka.toyama.jp i tateyama.toyama.jp i toga.toyama.jp i tonami.toyama.jp i toyama.toyama.jp i unazuki.toyama.jp i uozu.toyama.jp i yamada.toyama.jp i arida.wakayama.jp i aridagawa.wakayama.jp i gobo.wakayama.jp i hashimoto.wakayama.jp i hidaka.wakayama.jp i hirogawa.wakayama.jp i inami.wakayama.jp i iwade.wakayama.jp i kainan.wakayama.jp i kamitonda.wakayama.jp i katsuragi.wakayama.jp i kimino.wakayama.jp i kinokawa.wakayama.jp i kitayama.wakayama.jp i koya.wakayama.jp i koza.wakayama.jp i kozagawa.wakayama.jp i kudoyama.wakayama.jp i kushimoto.wakayama.jp i mihama.wakayama.jp i misato.wakayama.jp i nachikatsuura.wakayama.jp i shingu.wakayama.jp i shirahama.wakayama.jp i taiji.wakayama.jp i tanabe.wakayama.jp i wakayama.wakayama.jp i yuasa.wakayama.jp i yura.wakayama.jp i asahi.yamagata.jp i funagata.yamagata.jp i higashine.yamagata.jp i iide.yamagata.jp i kahoku.yamagata.jp i kaminoyama.yamagata.jp i kaneyama.yamagata.jp i kawanishi.yamagata.jp i mamurogawa.yamagata.jp i mikawa.yamagata.jp i murayama.yamagata.jp i nagai.yamagata.jp i nakayama.yamagata.jp i nanyo.yamagata.jp i nishikawa.yamagata.jp i obanazawa.yamagata.jp i oe.yamagata.jp i oguni.yamagata.jp i ohkura.yamagata.jp i oishida.yamagata.jp i sagae.yamagata.jp i sakata.yamagata.jp i sakegawa.yamagata.jp i shinjo.yamagata.jp i shirataka.yamagata.jp i shonai.yamagata.jp i takahata.yamagata.jp i tendo.yamagata.jp i tozawa.yamagata.jp i tsuruoka.yamagata.jp i yamagata.yamagata.jp i yamanobe.yamagata.jp i yonezawa.yamagata.jp i yuza.yamagata.jp i abu.yamaguchi.jp i hagi.yamaguchi.jp i hikari.yamaguchi.jp i hofu.yamaguchi.jp i iwakuni.yamaguchi.jp i kudamatsu.yamaguchi.jp i mitou.yamaguchi.jp i nagato.yamaguchi.jp i oshima.yamaguchi.jp i shimonoseki.yamaguchi.jp i shunan.yamaguchi.jp i tabuse.yamaguchi.jp i tokuyama.yamaguchi.jp i toyota.yamaguchi.jp i ube.yamaguchi.jp i yuu.yamaguchi.jp i chuo.yamanashi.jp i doshi.yamanashi.jp i fuefuki.yamanashi.jp i fujikawa.yamanashi.jp i fujikawaguchiko.yamanashi.jp i fujiyoshida.yamanashi.jp i hayakawa.yamanashi.jp i hokuto.yamanashi.jp i ichikawamisato.yamanashi.jp i kai.yamanashi.jp i kofu.yamanashi.jp i koshu.yamanashi.jp i kosuge.yamanashi.jp i minami-alps.yamanashi.jp i minobu.yamanashi.jp i nakamichi.yamanashi.jp i nanbu.yamanashi.jp i narusawa.yamanashi.jp i nirasaki.yamanashi.jp i nishikatsura.yamanashi.jp i oshino.yamanashi.jp i otsuki.yamanashi.jp i showa.yamanashi.jp i tabayama.yamanashi.jp i tsuru.yamanashi.jp i uenohara.yamanashi.jp i yamanakako.yamanashi.jp i yamanashi.yamanashi.jp i ke w kg i org.kg i net.kg i com.kg i edu.kg i gov.kg i mil.kg i kh w ki i edu.ki i biz.ki i net.ki i org.ki i gov.ki i info.ki i com.ki i km i org.km i nom.km i gov.km i prd.km i tm.km i edu.km i mil.km i ass.km i com.km i coop.km i asso.km i presse.km i medecin.km i notaires.km i pharmaciens.km i veterinaire.km i gouv.km i kn i net.kn i org.kn i edu.kn i gov.kn i kp i com.kp i edu.kp i gov.kp i org.kp i rep.kp i tra.kp i kr i ac.kr i co.kr i es.kr i go.kr i hs.kr i kg.kr i mil.kr i ms.kr i ne.kr i or.kr i pe.kr i re.kr i sc.kr i busan.kr i chungbuk.kr i chungnam.kr i daegu.kr i daejeon.kr i gangwon.kr i gwangju.kr i gyeongbuk.kr i gyeonggi.kr i gyeongnam.kr i incheon.kr i jeju.kr i jeonbuk.kr i jeonnam.kr i seoul.kr i ulsan.kr i kw w ky i edu.ky i gov.ky i com.ky i org.ky i net.ky i kz i org.kz i edu.kz i net.kz i gov.kz i mil.kz i com.kz i la i int.la i net.la i info.la i edu.la i gov.la i per.la i com.la i org.la i lb i com.lb i edu.lb i gov.lb i net.lb i org.lb i lc i com.lc i net.lc i co.lc i org.lc i edu.lc i gov.lc i li i lk i gov.lk i sch.lk i net.lk i int.lk i com.lk i org.lk i edu.lk i ngo.lk i soc.lk i web.lk i ltd.lk i assn.lk i grp.lk i hotel.lk i lr i com.lr i edu.lr i gov.lr i org.lr i net.lr i ls i co.ls i org.ls i lt i gov.lt i lu i lv i com.lv i edu.lv i gov.lv i org.lv i mil.lv i id.lv i net.lv i asn.lv i conf.lv i ly i com.ly i net.ly i gov.ly i plc.ly i edu.ly i sch.ly i med.ly i org.ly i id.ly i ma i co.ma i net.ma i gov.ma i org.ma i ac.ma i press.ma i mc i tm.mc i asso.mc i md i me i co.me i net.me i org.me i edu.me i ac.me i gov.me i its.me i priv.me i mg i org.mg i nom.mg i gov.mg i prd.mg i tm.mg i edu.mg i mil.mg i com.mg i mh i mil i mk i com.mk i org.mk i net.mk i edu.mk i gov.mk i inf.mk i name.mk i ml i com.ml i edu.ml i gouv.ml i gov.ml i net.ml i org.ml i presse.ml i mm w mn i gov.mn i edu.mn i org.mn i mo i com.mo i net.mo i org.mo i edu.mo i gov.mo i mobi i mp i mq i mr i gov.mr i ms i com.ms i edu.ms i gov.ms i net.ms i org.ms i mt i com.mt i edu.mt i net.mt i org.mt i mu i com.mu i net.mu i org.mu i gov.mu i ac.mu i co.mu i or.mu i museum i academy.museum i agriculture.museum i air.museum i airguard.museum i alabama.museum i alaska.museum i amber.museum i ambulance.museum i american.museum i americana.museum i americanantiques.museum i americanart.museum i amsterdam.museum i and.museum i annefrank.museum i anthro.museum i anthropology.museum i antiques.museum i aquarium.museum i arboretum.museum i archaeological.museum i archaeology.museum i architecture.museum i art.museum i artanddesign.museum i artcenter.museum i artdeco.museum i arteducation.museum i artgallery.museum i arts.museum i artsandcrafts.museum i asmatart.museum i assassination.museum i assisi.museum i association.museum i astronomy.museum i atlanta.museum i austin.museum i australia.museum i automotive.museum i aviation.museum i axis.museum i badajoz.museum i baghdad.museum i bahn.museum i bale.museum i baltimore.museum i barcelona.museum i baseball.museum i basel.museum i baths.museum i bauern.museum i beauxarts.museum i beeldengeluid.museum i bellevue.museum i bergbau.museum i berkeley.museum i berlin.museum i bern.museum i bible.museum i bilbao.museum i bill.museum i birdart.museum i birthplace.museum i bonn.museum i boston.museum i botanical.museum i botanicalgarden.museum i botanicgarden.museum i botany.museum i brandywinevalley.museum i brasil.museum i bristol.museum i british.museum i britishcolumbia.museum i broadcast.museum i brunel.museum i brussel.museum i brussels.museum i bruxelles.museum i building.museum i burghof.museum i bus.museum i bushey.museum i cadaques.museum i california.museum i cambridge.museum i can.museum i canada.museum i capebreton.museum i carrier.museum i cartoonart.museum i casadelamoneda.museum i castle.museum i castres.museum i celtic.museum i center.museum i chattanooga.museum i cheltenham.museum i chesapeakebay.museum i chicago.museum i children.museum i childrens.museum i childrensgarden.museum i chiropractic.museum i chocolate.museum i christiansburg.museum i cincinnati.museum i cinema.museum i circus.museum i civilisation.museum i civilization.museum i civilwar.museum i clinton.museum i clock.museum i coal.museum i coastaldefence.museum i cody.museum i coldwar.museum i collection.museum i colonialwilliamsburg.museum i coloradoplateau.museum i columbia.museum i columbus.museum i communication.museum i communications.museum i community.museum i computer.museum i computerhistory.museum i comunicações.museum i contemporary.museum i contemporaryart.museum i convent.museum i copenhagen.museum i corporation.museum i correios-e-telecomunicações.museum i corvette.museum i costume.museum i countryestate.museum i county.museum i crafts.museum i cranbrook.museum i creation.museum i cultural.museum i culturalcenter.museum i culture.museum i cyber.museum i cymru.museum i dali.museum i dallas.museum i database.museum i ddr.museum i decorativearts.museum i delaware.museum i delmenhorst.museum i denmark.museum i depot.museum i design.museum i detroit.museum i dinosaur.museum i discovery.museum i dolls.museum i donostia.museum i durham.museum i eastafrica.museum i eastcoast.museum i education.museum i educational.museum i egyptian.museum i eisenbahn.museum i elburg.museum i elvendrell.museum i embroidery.museum i encyclopedic.museum i england.museum i entomology.museum i environment.museum i environmentalconservation.museum i epilepsy.museum i essex.museum i estate.museum i ethnology.museum i exeter.museum i exhibition.museum i family.museum i farm.museum i farmequipment.museum i farmers.museum i farmstead.museum i field.museum i figueres.museum i filatelia.museum i film.museum i fineart.museum i finearts.museum i finland.museum i flanders.museum i florida.museum i force.museum i fortmissoula.museum i fortworth.museum i foundation.museum i francaise.museum i frankfurt.museum i franziskaner.museum i freemasonry.museum i freiburg.museum i fribourg.museum i frog.museum i fundacio.museum i furniture.museum i gallery.museum i garden.museum i gateway.museum i geelvinck.museum i gemological.museum i geology.museum i georgia.museum i giessen.museum i glas.museum i glass.museum i gorge.museum i grandrapids.museum i graz.museum i guernsey.museum i halloffame.museum i hamburg.museum i handson.museum i harvestcelebration.museum i hawaii.museum i health.museum i heimatunduhren.museum i hellas.museum i helsinki.museum i hembygdsforbund.museum i heritage.museum i histoire.museum i historical.museum i historicalsociety.museum i historichouses.museum i historisch.museum i historisches.museum i history.museum i historyofscience.museum i horology.museum i house.museum i humanities.museum i illustration.museum i imageandsound.museum i indian.museum i indiana.museum i indianapolis.museum i indianmarket.museum i intelligence.museum i interactive.museum i iraq.museum i iron.museum i isleofman.museum i jamison.museum i jefferson.museum i jerusalem.museum i jewelry.museum i jewish.museum i jewishart.museum i jfk.museum i journalism.museum i judaica.museum i judygarland.museum i juedisches.museum i juif.museum i karate.museum i karikatur.museum i kids.museum i koebenhavn.museum i koeln.museum i kunst.museum i kunstsammlung.museum i kunstunddesign.museum i labor.museum i labour.museum i lajolla.museum i lancashire.museum i landes.museum i lans.museum i läns.museum i larsson.museum i lewismiller.museum i lincoln.museum i linz.museum i living.museum i livinghistory.museum i localhistory.museum i london.museum i losangeles.museum i louvre.museum i loyalist.museum i lucerne.museum i luxembourg.museum i luzern.museum i mad.museum i madrid.museum i mallorca.museum i manchester.museum i mansion.museum i mansions.museum i manx.museum i marburg.museum i maritime.museum i maritimo.museum i maryland.museum i marylhurst.museum i media.museum i medical.museum i medizinhistorisches.museum i meeres.museum i memorial.museum i mesaverde.museum i michigan.museum i midatlantic.museum i military.museum i mill.museum i miners.museum i mining.museum i minnesota.museum i missile.museum i missoula.museum i modern.museum i moma.museum i money.museum i monmouth.museum i monticello.museum i montreal.museum i moscow.museum i motorcycle.museum i muenchen.museum i muenster.museum i mulhouse.museum i muncie.museum i museet.museum i museumcenter.museum i museumvereniging.museum i music.museum i national.museum i nationalfirearms.museum i nationalheritage.museum i nativeamerican.museum i naturalhistory.museum i naturalhistorymuseum.museum i naturalsciences.museum i nature.museum i naturhistorisches.museum i natuurwetenschappen.museum i naumburg.museum i naval.museum i nebraska.museum i neues.museum i newhampshire.museum i newjersey.museum i newmexico.museum i newport.museum i newspaper.museum i newyork.museum i niepce.museum i norfolk.museum i north.museum i nrw.museum i nuernberg.museum i nuremberg.museum i nyc.museum i nyny.museum i oceanographic.museum i oceanographique.museum i omaha.museum i online.museum i ontario.museum i openair.museum i oregon.museum i oregontrail.museum i otago.museum i oxford.museum i pacific.museum i paderborn.museum i palace.museum i paleo.museum i palmsprings.museum i panama.museum i paris.museum i pasadena.museum i pharmacy.museum i philadelphia.museum i philadelphiaarea.museum i philately.museum i phoenix.museum i photography.museum i pilots.museum i pittsburgh.museum i planetarium.museum i plantation.museum i plants.museum i plaza.museum i portal.museum i portland.museum i portlligat.museum i posts-and-telecommunications.museum i preservation.museum i presidio.museum i press.museum i project.museum i public.museum i pubol.museum i quebec.museum i railroad.museum i railway.museum i research.museum i resistance.museum i riodejaneiro.museum i rochester.museum i rockart.museum i roma.museum i russia.museum i saintlouis.museum i salem.museum i salvadordali.museum i salzburg.museum i sandiego.museum i sanfrancisco.museum i santabarbara.museum i santacruz.museum i santafe.museum i saskatchewan.museum i satx.museum i savannahga.museum i schlesisches.museum i schoenbrunn.museum i schokoladen.museum i school.museum i schweiz.museum i science.museum i scienceandhistory.museum i scienceandindustry.museum i sciencecenter.museum i sciencecenters.museum i science-fiction.museum i sciencehistory.museum i sciences.museum i sciencesnaturelles.museum i scotland.museum i seaport.museum i settlement.museum i settlers.museum i shell.museum i sherbrooke.museum i sibenik.museum i silk.museum i ski.museum i skole.museum i society.museum i sologne.museum i soundandvision.museum i southcarolina.museum i southwest.museum i space.museum i spy.museum i square.museum i stadt.museum i stalbans.museum i starnberg.museum i state.museum i stateofdelaware.museum i station.museum i steam.museum i steiermark.museum i stjohn.museum i stockholm.museum i stpetersburg.museum i stuttgart.museum i suisse.museum i surgeonshall.museum i surrey.museum i svizzera.museum i sweden.museum i sydney.museum i tank.museum i tcm.museum i technology.museum i telekommunikation.museum i television.museum i texas.museum i textile.museum i theater.museum i time.museum i timekeeping.museum i topology.museum i torino.museum i touch.museum i town.museum i transport.museum i tree.museum i trolley.museum i trust.museum i trustee.museum i uhren.museum i ulm.museum i undersea.museum i university.museum i usa.museum i usantiques.museum i usarts.museum i uscountryestate.museum i usculture.museum i usdecorativearts.museum i usgarden.museum i ushistory.museum i ushuaia.museum i uslivinghistory.museum i utah.museum i uvic.museum i valley.museum i vantaa.museum i versailles.museum i viking.museum i village.museum i virginia.museum i virtual.museum i virtuel.museum i vlaanderen.museum i volkenkunde.museum i wales.museum i wallonie.museum i war.museum i washingtondc.museum i watchandclock.museum i watch-and-clock.museum i western.museum i westfalen.museum i whaling.museum i wildlife.museum i williamsburg.museum i windmill.museum i workshop.museum i york.museum i yorkshire.museum i yosemite.museum i youth.museum i zoological.museum i zoology.museum i иком.museum i mv i aero.mv i biz.mv i com.mv i coop.mv i edu.mv i gov.mv i info.mv i int.mv i mil.mv i museum.mv i name.mv i net.mv i org.mv i pro.mv i mw i ac.mw i biz.mw i co.mw i com.mw i coop.mw i edu.mw i gov.mw i int.mw i museum.mw i net.mw i org.mw i mx i com.mx i org.mx i gob.mx i edu.mx i net.mx i my i com.my i net.my i org.my i gov.my i edu.my i mil.my i name.my i mz w teledata.mz e na i info.na i pro.na i name.na i school.na i or.na i dr.na i us.na i mx.na i ca.na i in.na i cc.na i tv.na i ws.na i mobi.na i co.na i com.na i org.na i name i nc i asso.nc i ne i net i nf i com.nf i net.nf i per.nf i rec.nf i web.nf i arts.nf i firm.nf i info.nf i other.nf i store.nf i ng i com.ng i edu.ng i name.ng i net.ng i org.ng i sch.ng i gov.ng i mil.ng i mobi.ng i ni w nl i bv.nl i no i fhs.no i vgs.no i fylkesbibl.no i folkebibl.no i museum.no i idrett.no i priv.no i mil.no i stat.no i dep.no i kommune.no i herad.no i aa.no i ah.no i bu.no i fm.no i hl.no i hm.no i jan-mayen.no i mr.no i nl.no i nt.no i of.no i ol.no i oslo.no i rl.no i sf.no i st.no i svalbard.no i tm.no i tr.no i va.no i vf.no i gs.aa.no i gs.ah.no i gs.bu.no i gs.fm.no i gs.hl.no i gs.hm.no i gs.jan-mayen.no i gs.mr.no i gs.nl.no i gs.nt.no i gs.of.no i gs.ol.no i gs.oslo.no i gs.rl.no i gs.sf.no i gs.st.no i gs.svalbard.no i gs.tm.no i gs.tr.no i gs.va.no i gs.vf.no i akrehamn.no i åkrehamn.no i algard.no i ålgård.no i arna.no i brumunddal.no i bryne.no i bronnoysund.no i brønnøysund.no i drobak.no i drøbak.no i egersund.no i fetsund.no i floro.no i florø.no i fredrikstad.no i hokksund.no i honefoss.no i hønefoss.no i jessheim.no i jorpeland.no i jørpeland.no i kirkenes.no i kopervik.no i krokstadelva.no i langevag.no i langevåg.no i leirvik.no i mjondalen.no i mjøndalen.no i mo-i-rana.no i mosjoen.no i mosjøen.no i nesoddtangen.no i orkanger.no i osoyro.no i osøyro.no i raholt.no i råholt.no i sandnessjoen.no i sandnessjøen.no i skedsmokorset.no i slattum.no i spjelkavik.no i stathelle.no i stavern.no i stjordalshalsen.no i stjørdalshalsen.no i tananger.no i tranby.no i vossevangen.no i afjord.no i åfjord.no i agdenes.no i al.no i ål.no i alesund.no i ålesund.no i alstahaug.no i alta.no i áltá.no i alaheadju.no i álaheadju.no i alvdal.no i amli.no i åmli.no i amot.no i åmot.no i andebu.no i andoy.no i andøy.no i andasuolo.no i ardal.no i årdal.no i aremark.no i arendal.no i ås.no i aseral.no i åseral.no i asker.no i askim.no i askvoll.no i askoy.no i askøy.no i asnes.no i åsnes.no i audnedaln.no i aukra.no i aure.no i aurland.no i aurskog-holand.no i aurskog-høland.no i austevoll.no i austrheim.no i averoy.no i averøy.no i balestrand.no i ballangen.no i balat.no i bálát.no i balsfjord.no i bahccavuotna.no i báhccavuotna.no i bamble.no i bardu.no i beardu.no i beiarn.no i bajddar.no i bájddar.no i baidar.no i báidár.no i berg.no i bergen.no i berlevag.no i berlevåg.no i bearalvahki.no i bearalváhki.no i bindal.no i birkenes.no i bjarkoy.no i bjarkøy.no i bjerkreim.no i bjugn.no i bodo.no i bodø.no i badaddja.no i bådåddjå.no i budejju.no i bokn.no i bremanger.no i bronnoy.no i brønnøy.no i bygland.no i bykle.no i barum.no i bærum.no i bo.telemark.no i bø.telemark.no i bo.nordland.no i bø.nordland.no i bievat.no i bievát.no i bomlo.no i bømlo.no i batsfjord.no i båtsfjord.no i bahcavuotna.no i báhcavuotna.no i dovre.no i drammen.no i drangedal.no i dyroy.no i dyrøy.no i donna.no i dønna.no i eid.no i eidfjord.no i eidsberg.no i eidskog.no i eidsvoll.no i eigersund.no i elverum.no i enebakk.no i engerdal.no i etne.no i etnedal.no i evenes.no i evenassi.no i evenášši.no i evje-og-hornnes.no i farsund.no i fauske.no i fuossko.no i fuoisku.no i fedje.no i fet.no i finnoy.no i finnøy.no i fitjar.no i fjaler.no i fjell.no i flakstad.no i flatanger.no i flekkefjord.no i flesberg.no i flora.no i fla.no i flå.no i folldal.no i forsand.no i fosnes.no i frei.no i frogn.no i froland.no i frosta.no i frana.no i fræna.no i froya.no i frøya.no i fusa.no i fyresdal.no i forde.no i førde.no i gamvik.no i gangaviika.no i gáŋgaviika.no i gaular.no i gausdal.no i gildeskal.no i gildeskål.no i giske.no i gjemnes.no i gjerdrum.no i gjerstad.no i gjesdal.no i gjovik.no i gjøvik.no i gloppen.no i gol.no i gran.no i grane.no i granvin.no i gratangen.no i grimstad.no i grong.no i kraanghke.no i kråanghke.no i grue.no i gulen.no i hadsel.no i halden.no i halsa.no i hamar.no i hamaroy.no i habmer.no i hábmer.no i hapmir.no i hápmir.no i hammerfest.no i hammarfeasta.no i hámmárfeasta.no i haram.no i hareid.no i harstad.no i hasvik.no i aknoluokta.no i ákŋoluokta.no i hattfjelldal.no i aarborte.no i haugesund.no i hemne.no i hemnes.no i hemsedal.no i heroy.more-og-romsdal.no i herøy.møre-og-romsdal.no i heroy.nordland.no i herøy.nordland.no i hitra.no i hjartdal.no i hjelmeland.no i hobol.no i hobøl.no i hof.no i hol.no i hole.no i holmestrand.no i holtalen.no i holtålen.no i hornindal.no i horten.no i hurdal.no i hurum.no i hvaler.no i hyllestad.no i hagebostad.no i hægebostad.no i hoyanger.no i høyanger.no i hoylandet.no i høylandet.no i ha.no i hå.no i ibestad.no i inderoy.no i inderøy.no i iveland.no i jevnaker.no i jondal.no i jolster.no i jølster.no i karasjok.no i karasjohka.no i kárášjohka.no i karlsoy.no i galsa.no i gálsá.no i karmoy.no i karmøy.no i kautokeino.no i guovdageaidnu.no i klepp.no i klabu.no i klæbu.no i kongsberg.no i kongsvinger.no i kragero.no i kragerø.no i kristiansand.no i kristiansund.no i krodsherad.no i krødsherad.no i kvalsund.no i rahkkeravju.no i ráhkkerávju.no i kvam.no i kvinesdal.no i kvinnherad.no i kviteseid.no i kvitsoy.no i kvitsøy.no i kvafjord.no i kvæfjord.no i giehtavuoatna.no i kvanangen.no i kvænangen.no i navuotna.no i návuotna.no i kafjord.no i kåfjord.no i gaivuotna.no i gáivuotna.no i larvik.no i lavangen.no i lavagis.no i loabat.no i loabát.no i lebesby.no i davvesiida.no i leikanger.no i leirfjord.no i leka.no i leksvik.no i lenvik.no i leangaviika.no i leaŋgaviika.no i lesja.no i levanger.no i lier.no i lierne.no i lillehammer.no i lillesand.no i lindesnes.no i lindas.no i lindås.no i lom.no i loppa.no i lahppi.no i láhppi.no i lund.no i lunner.no i luroy.no i lurøy.no i luster.no i lyngdal.no i lyngen.no i ivgu.no i lardal.no i lerdal.no i lærdal.no i lodingen.no i lødingen.no i lorenskog.no i lørenskog.no i loten.no i løten.no i malvik.no i masoy.no i måsøy.no i muosat.no i muosát.no i mandal.no i marker.no i marnardal.no i masfjorden.no i meland.no i meldal.no i melhus.no i meloy.no i meløy.no i meraker.no i meråker.no i moareke.no i moåreke.no i midsund.no i midtre-gauldal.no i modalen.no i modum.no i molde.no i moskenes.no i moss.no i mosvik.no i malselv.no i målselv.no i malatvuopmi.no i málatvuopmi.no i namdalseid.no i aejrie.no i namsos.no i namsskogan.no i naamesjevuemie.no i nååmesjevuemie.no i laakesvuemie.no i nannestad.no i narvik.no i narviika.no i naustdal.no i nedre-eiker.no i nes.akershus.no i nes.buskerud.no i nesna.no i nesodden.no i nesseby.no i unjarga.no i unjárga.no i nesset.no i nissedal.no i nittedal.no i nord-aurdal.no i nord-fron.no i nord-odal.no i norddal.no i nordkapp.no i davvenjarga.no i davvenjárga.no i nordre-land.no i nordreisa.no i raisa.no i ráisa.no i nore-og-uvdal.no i notodden.no i naroy.no i nærøy.no i notteroy.no i nøtterøy.no i odda.no i oksnes.no i øksnes.no i oppdal.no i oppegard.no i oppegård.no i orkdal.no i orland.no i ørland.no i orskog.no i ørskog.no i orsta.no i ørsta.no i os.hedmark.no i os.hordaland.no i osen.no i osteroy.no i osterøy.no i ostre-toten.no i østre-toten.no i overhalla.no i ovre-eiker.no i øvre-eiker.no i oyer.no i øyer.no i oygarden.no i øygarden.no i oystre-slidre.no i øystre-slidre.no i porsanger.no i porsangu.no i porsáŋgu.no i porsgrunn.no i radoy.no i radøy.no i rakkestad.no i rana.no i ruovat.no i randaberg.no i rauma.no i rendalen.no i rennebu.no i rennesoy.no i rennesøy.no i rindal.no i ringebu.no i ringerike.no i ringsaker.no i rissa.no i risor.no i risør.no i roan.no i rollag.no i rygge.no i ralingen.no i rælingen.no i rodoy.no i rødøy.no i romskog.no i rømskog.no i roros.no i røros.no i rost.no i røst.no i royken.no i røyken.no i royrvik.no i røyrvik.no i rade.no i råde.no i salangen.no i siellak.no i saltdal.no i salat.no i sálát.no i sálat.no i samnanger.no i sande.more-og-romsdal.no i sande.møre-og-romsdal.no i sande.vestfold.no i sandefjord.no i sandnes.no i sandoy.no i sandøy.no i sarpsborg.no i sauda.no i sauherad.no i sel.no i selbu.no i selje.no i seljord.no i sigdal.no i siljan.no i sirdal.no i skaun.no i skedsmo.no i ski.no i skien.no i skiptvet.no i skjervoy.no i skjervøy.no i skierva.no i skiervá.no i skjak.no i skjåk.no i skodje.no i skanland.no i skånland.no i skanit.no i skánit.no i smola.no i smøla.no i snillfjord.no i snasa.no i snåsa.no i snoasa.no i snaase.no i snåase.no i sogndal.no i sokndal.no i sola.no i solund.no i songdalen.no i sortland.no i spydeberg.no i stange.no i stavanger.no i steigen.no i steinkjer.no i stjordal.no i stjørdal.no i stokke.no i stor-elvdal.no i stord.no i stordal.no i storfjord.no i omasvuotna.no i strand.no i stranda.no i stryn.no i sula.no i suldal.no i sund.no i sunndal.no i surnadal.no i sveio.no i svelvik.no i sykkylven.no i sogne.no i søgne.no i somna.no i sømna.no i sondre-land.no i søndre-land.no i sor-aurdal.no i sør-aurdal.no i sor-fron.no i sør-fron.no i sor-odal.no i sør-odal.no i sor-varanger.no i sør-varanger.no i matta-varjjat.no i mátta-várjjat.no i sorfold.no i sørfold.no i sorreisa.no i sørreisa.no i sorum.no i sørum.no i tana.no i deatnu.no i time.no i tingvoll.no i tinn.no i tjeldsund.no i dielddanuorri.no i tjome.no i tjøme.no i tokke.no i tolga.no i torsken.no i tranoy.no i tranøy.no i tromso.no i tromsø.no i tromsa.no i romsa.no i trondheim.no i troandin.no i trysil.no i trana.no i træna.no i trogstad.no i trøgstad.no i tvedestrand.no i tydal.no i tynset.no i tysfjord.no i divtasvuodna.no i divttasvuotna.no i tysnes.no i tysvar.no i tysvær.no i tonsberg.no i tønsberg.no i ullensaker.no i ullensvang.no i ulvik.no i utsira.no i vadso.no i vadsø.no i cahcesuolo.no i čáhcesuolo.no i vaksdal.no i valle.no i vang.no i vanylven.no i vardo.no i vardø.no i varggat.no i várggát.no i vefsn.no i vaapste.no i vega.no i vegarshei.no i vegårshei.no i vennesla.no i verdal.no i verran.no i vestby.no i vestnes.no i vestre-slidre.no i vestre-toten.no i vestvagoy.no i vestvågøy.no i vevelstad.no i vik.no i vikna.no i vindafjord.no i volda.no i voss.no i varoy.no i værøy.no i vagan.no i vågan.no i voagat.no i vagsoy.no i vågsøy.no i vaga.no i vågå.no i valer.ostfold.no i våler.østfold.no i valer.hedmark.no i våler.hedmark.no i np w nr i biz.nr i info.nr i gov.nr i edu.nr i org.nr i net.nr i com.nr i nu i nz i ac.nz i co.nz i cri.nz i geek.nz i gen.nz i govt.nz i health.nz i iwi.nz i kiwi.nz i maori.nz i mil.nz i māori.nz i net.nz i org.nz i parliament.nz i school.nz i om i co.om i com.om i edu.om i gov.om i med.om i museum.om i net.om i org.om i pro.om i org i pa i ac.pa i gob.pa i com.pa i org.pa i sld.pa i edu.pa i net.pa i ing.pa i abo.pa i med.pa i nom.pa i pe i edu.pe i gob.pe i nom.pe i mil.pe i org.pe i com.pe i net.pe i pf i com.pf i org.pf i edu.pf i pg w ph i com.ph i net.ph i org.ph i gov.ph i edu.ph i ngo.ph i mil.ph i i.ph i pk i com.pk i net.pk i edu.pk i org.pk i fam.pk i biz.pk i web.pk i gov.pk i gob.pk i gok.pk i gon.pk i gop.pk i gos.pk i info.pk i pl i aid.pl i agro.pl i atm.pl i auto.pl i biz.pl i com.pl i edu.pl i gmina.pl i gsm.pl i info.pl i mail.pl i miasta.pl i media.pl i mil.pl i net.pl i nieruchomosci.pl i nom.pl i org.pl i pc.pl i powiat.pl i priv.pl i realestate.pl i rel.pl i sex.pl i shop.pl i sklep.pl i sos.pl i szkola.pl i targi.pl i tm.pl i tourism.pl i travel.pl i turystyka.pl i 6bone.pl i art.pl i mbone.pl i gov.pl i uw.gov.pl i um.gov.pl i ug.gov.pl i upow.gov.pl i starostwo.gov.pl i so.gov.pl i sr.gov.pl i po.gov.pl i pa.gov.pl i ngo.pl i irc.pl i usenet.pl i augustow.pl i babia-gora.pl i bedzin.pl i beskidy.pl i bialowieza.pl i bialystok.pl i bielawa.pl i bieszczady.pl i boleslawiec.pl i bydgoszcz.pl i bytom.pl i cieszyn.pl i czeladz.pl i czest.pl i dlugoleka.pl i elblag.pl i elk.pl i glogow.pl i gniezno.pl i gorlice.pl i grajewo.pl i ilawa.pl i jaworzno.pl i jelenia-gora.pl i jgora.pl i kalisz.pl i kazimierz-dolny.pl i karpacz.pl i kartuzy.pl i kaszuby.pl i katowice.pl i kepno.pl i ketrzyn.pl i klodzko.pl i kobierzyce.pl i kolobrzeg.pl i konin.pl i konskowola.pl i kutno.pl i lapy.pl i lebork.pl i legnica.pl i lezajsk.pl i limanowa.pl i lomza.pl i lowicz.pl i lubin.pl i lukow.pl i malbork.pl i malopolska.pl i mazowsze.pl i mazury.pl i mielec.pl i mielno.pl i mragowo.pl i naklo.pl i nowaruda.pl i nysa.pl i olawa.pl i olecko.pl i olkusz.pl i olsztyn.pl i opoczno.pl i opole.pl i ostroda.pl i ostroleka.pl i ostrowiec.pl i ostrowwlkp.pl i pila.pl i pisz.pl i podhale.pl i podlasie.pl i polkowice.pl i pomorze.pl i pomorskie.pl i prochowice.pl i pruszkow.pl i przeworsk.pl i pulawy.pl i radom.pl i rawa-maz.pl i rybnik.pl i rzeszow.pl i sanok.pl i sejny.pl i siedlce.pl i slask.pl i slupsk.pl i sosnowiec.pl i stalowa-wola.pl i skoczow.pl i starachowice.pl i stargard.pl i suwalki.pl i swidnica.pl i swiebodzin.pl i swinoujscie.pl i szczecin.pl i szczytno.pl i tarnobrzeg.pl i tgory.pl i turek.pl i tychy.pl i ustka.pl i walbrzych.pl i warmia.pl i warszawa.pl i waw.pl i wegrow.pl i wielun.pl i wlocl.pl i wloclawek.pl i wodzislaw.pl i wolomin.pl i wroclaw.pl i zachpomor.pl i zagan.pl i zarow.pl i zgora.pl i zgorzelec.pl i gda.pl i gdansk.pl i gdynia.pl i med.pl i sopot.pl i gliwice.pl i krakow.pl i poznan.pl i wroc.pl i zakopane.pl i pm i pn i gov.pn i co.pn i org.pn i edu.pn i net.pn i post i pr i com.pr i net.pr i org.pr i gov.pr i edu.pr i isla.pr i pro.pr i biz.pr i info.pr i name.pr i est.pr i prof.pr i ac.pr i pro i aca.pro i bar.pro i cpa.pro i jur.pro i law.pro i med.pro i eng.pro i ps i edu.ps i gov.ps i sec.ps i plo.ps i com.ps i org.ps i net.ps i pt i net.pt i gov.pt i org.pt i edu.pt i int.pt i publ.pt i com.pt i nome.pt i pw i co.pw i ne.pw i or.pw i ed.pw i go.pw i belau.pw i py i com.py i coop.py i edu.py i gov.py i mil.py i net.py i org.py i qa i com.qa i edu.qa i gov.qa i mil.qa i name.qa i net.qa i org.qa i sch.qa i re i com.re i asso.re i nom.re i ro i com.ro i org.ro i tm.ro i nt.ro i nom.ro i info.ro i rec.ro i arts.ro i firm.ro i store.ro i www.ro i rs i co.rs i org.rs i edu.rs i ac.rs i gov.rs i in.rs i ru i ac.ru i com.ru i edu.ru i int.ru i net.ru i org.ru i pp.ru i adygeya.ru i altai.ru i amur.ru i arkhangelsk.ru i astrakhan.ru i bashkiria.ru i belgorod.ru i bir.ru i bryansk.ru i buryatia.ru i cbg.ru i chel.ru i chelyabinsk.ru i chita.ru i chukotka.ru i chuvashia.ru i dagestan.ru i dudinka.ru i e-burg.ru i grozny.ru i irkutsk.ru i ivanovo.ru i izhevsk.ru i jar.ru i joshkar-ola.ru i kalmykia.ru i kaluga.ru i kamchatka.ru i karelia.ru i kazan.ru i kchr.ru i kemerovo.ru i khabarovsk.ru i khakassia.ru i khv.ru i kirov.ru i koenig.ru i komi.ru i kostroma.ru i krasnoyarsk.ru i kuban.ru i kurgan.ru i kursk.ru i lipetsk.ru i magadan.ru i mari.ru i mari-el.ru i marine.ru i mordovia.ru i mosreg.ru i msk.ru i murmansk.ru i nalchik.ru i nnov.ru i nov.ru i novosibirsk.ru i nsk.ru i omsk.ru i orenburg.ru i oryol.ru i palana.ru i penza.ru i perm.ru i ptz.ru i rnd.ru i ryazan.ru i sakhalin.ru i samara.ru i saratov.ru i simbirsk.ru i smolensk.ru i spb.ru i stavropol.ru i stv.ru i surgut.ru i tambov.ru i tatarstan.ru i tom.ru i tomsk.ru i tsaritsyn.ru i tsk.ru i tula.ru i tuva.ru i tver.ru i tyumen.ru i udm.ru i udmurtia.ru i ulan-ude.ru i vladikavkaz.ru i vladimir.ru i vladivostok.ru i volgograd.ru i vologda.ru i voronezh.ru i vrn.ru i vyatka.ru i yakutia.ru i yamal.ru i yaroslavl.ru i yekaterinburg.ru i yuzhno-sakhalinsk.ru i amursk.ru i baikal.ru i cmw.ru i fareast.ru i jamal.ru i kms.ru i k-uralsk.ru i kustanai.ru i kuzbass.ru i magnitka.ru i mytis.ru i nakhodka.ru i nkz.ru i norilsk.ru i oskol.ru i pyatigorsk.ru i rubtsovsk.ru i snz.ru i syzran.ru i vdonsk.ru i zgrad.ru i gov.ru i mil.ru i test.ru i rw i gov.rw i net.rw i edu.rw i ac.rw i com.rw i co.rw i int.rw i mil.rw i gouv.rw i sa i com.sa i net.sa i org.sa i gov.sa i med.sa i pub.sa i edu.sa i sch.sa i sb i com.sb i edu.sb i gov.sb i net.sb i org.sb i sc i com.sc i gov.sc i net.sc i org.sc i edu.sc i sd i com.sd i net.sd i org.sd i edu.sd i med.sd i tv.sd i gov.sd i info.sd i se i a.se i ac.se i b.se i bd.se i brand.se i c.se i d.se i e.se i f.se i fh.se i fhsk.se i fhv.se i g.se i h.se i i.se i k.se i komforb.se i kommunalforbund.se i komvux.se i l.se i lanbib.se i m.se i n.se i naturbruksgymn.se i o.se i org.se i p.se i parti.se i pp.se i press.se i r.se i s.se i t.se i tm.se i u.se i w.se i x.se i y.se i z.se i sg i com.sg i net.sg i org.sg i gov.sg i edu.sg i per.sg i sh i com.sh i net.sh i gov.sh i org.sh i mil.sh i si i sj i sk i sl i com.sl i net.sl i edu.sl i gov.sl i org.sl i sm i sn i art.sn i com.sn i edu.sn i gouv.sn i org.sn i perso.sn i univ.sn i so i com.so i net.so i org.so i sr i st i co.st i com.st i consulado.st i edu.st i embaixada.st i gov.st i mil.st i net.st i org.st i principe.st i saotome.st i store.st i su i sv i com.sv i edu.sv i gob.sv i org.sv i red.sv i sx i gov.sx i sy i edu.sy i gov.sy i net.sy i mil.sy i com.sy i org.sy i sz i co.sz i ac.sz i org.sz i tc i td i tel i tf i tg i th i ac.th i co.th i go.th i in.th i mi.th i net.th i or.th i tj i ac.tj i biz.tj i co.tj i com.tj i edu.tj i go.tj i gov.tj i int.tj i mil.tj i name.tj i net.tj i nic.tj i org.tj i test.tj i web.tj i tk i tl i gov.tl i tm i com.tm i co.tm i org.tm i net.tm i nom.tm i gov.tm i mil.tm i edu.tm i tn i com.tn i ens.tn i fin.tn i gov.tn i ind.tn i intl.tn i nat.tn i net.tn i org.tn i info.tn i perso.tn i tourism.tn i edunet.tn i rnrt.tn i rns.tn i rnu.tn i mincom.tn i agrinet.tn i defense.tn i turen.tn i to i com.to i gov.to i net.to i org.to i edu.to i mil.to i tp i tr w nic.tr e gov.nc.tr i travel i tt i co.tt i com.tt i org.tt i net.tt i biz.tt i info.tt i pro.tt i int.tt i coop.tt i jobs.tt i mobi.tt i travel.tt i museum.tt i aero.tt i name.tt i gov.tt i edu.tt i tv i tw i edu.tw i gov.tw i mil.tw i com.tw i net.tw i org.tw i idv.tw i game.tw i ebiz.tw i club.tw i 網路.tw i 組織.tw i 商業.tw i tz i ac.tz i co.tz i go.tz i hotel.tz i info.tz i me.tz i mil.tz i mobi.tz i ne.tz i or.tz i sc.tz i tv.tz i ua i com.ua i edu.ua i gov.ua i in.ua i net.ua i org.ua i cherkassy.ua i cherkasy.ua i chernigov.ua i chernihiv.ua i chernivtsi.ua i chernovtsy.ua i ck.ua i cn.ua i cr.ua i crimea.ua i cv.ua i dn.ua i dnepropetrovsk.ua i dnipropetrovsk.ua i dominic.ua i donetsk.ua i dp.ua i if.ua i ivano-frankivsk.ua i kh.ua i kharkiv.ua i kharkov.ua i kherson.ua i khmelnitskiy.ua i khmelnytskyi.ua i kiev.ua i kirovograd.ua i km.ua i kr.ua i krym.ua i ks.ua i kv.ua i kyiv.ua i lg.ua i lt.ua i lugansk.ua i lutsk.ua i lv.ua i lviv.ua i mk.ua i mykolaiv.ua i nikolaev.ua i od.ua i odesa.ua i odessa.ua i pl.ua i poltava.ua i rivne.ua i rovno.ua i rv.ua i sb.ua i sebastopol.ua i sevastopol.ua i sm.ua i sumy.ua i te.ua i ternopil.ua i uz.ua i uzhgorod.ua i vinnica.ua i vinnytsia.ua i vn.ua i volyn.ua i yalta.ua i zaporizhzhe.ua i zaporizhzhia.ua i zhitomir.ua i zhytomyr.ua i zp.ua i zt.ua i co.ua i pp.ua i ug i co.ug i or.ug i ac.ug i sc.ug i go.ug i ne.ug i com.ug i org.ug i uk i ac.uk i co.uk i gov.uk i ltd.uk i me.uk i net.uk i nhs.uk i org.uk i plc.uk i police.uk i sch.uk w us i dni.us i fed.us i isa.us i kids.us i nsn.us i ak.us i al.us i ar.us i as.us i az.us i ca.us i co.us i ct.us i dc.us i de.us i fl.us i ga.us i gu.us i hi.us i ia.us i id.us i il.us i in.us i ks.us i ky.us i la.us i ma.us i md.us i me.us i mi.us i mn.us i mo.us i ms.us i mt.us i nc.us i nd.us i ne.us i nh.us i nj.us i nm.us i nv.us i ny.us i oh.us i ok.us i or.us i pa.us i pr.us i ri.us i sc.us i sd.us i tn.us i tx.us i ut.us i vi.us i vt.us i va.us i wa.us i wi.us i wv.us i wy.us i k12.ak.us i k12.al.us i k12.ar.us i k12.as.us i k12.az.us i k12.ca.us i k12.co.us i k12.ct.us i k12.dc.us i k12.de.us i k12.fl.us i k12.ga.us i k12.gu.us i k12.ia.us i k12.id.us i k12.il.us i k12.in.us i k12.ks.us i k12.ky.us i k12.la.us i k12.ma.us i k12.md.us i k12.me.us i k12.mi.us i k12.mn.us i k12.mo.us i k12.ms.us i k12.mt.us i k12.nc.us i k12.nd.us i k12.ne.us i k12.nh.us i k12.nj.us i k12.nm.us i k12.nv.us i k12.ny.us i k12.oh.us i k12.ok.us i k12.or.us i k12.pa.us i k12.pr.us i k12.ri.us i k12.sc.us i k12.tn.us i k12.tx.us i k12.ut.us i k12.vi.us i k12.vt.us i k12.va.us i k12.wa.us i k12.wi.us i k12.wy.us i cc.ak.us i cc.al.us i cc.ar.us i cc.as.us i cc.az.us i cc.ca.us i cc.co.us i cc.ct.us i cc.dc.us i cc.de.us i cc.fl.us i cc.ga.us i cc.gu.us i cc.hi.us i cc.ia.us i cc.id.us i cc.il.us i cc.in.us i cc.ks.us i cc.ky.us i cc.la.us i cc.ma.us i cc.md.us i cc.me.us i cc.mi.us i cc.mn.us i cc.mo.us i cc.ms.us i cc.mt.us i cc.nc.us i cc.nd.us i cc.ne.us i cc.nh.us i cc.nj.us i cc.nm.us i cc.nv.us i cc.ny.us i cc.oh.us i cc.ok.us i cc.or.us i cc.pa.us i cc.pr.us i cc.ri.us i cc.sc.us i cc.sd.us i cc.tn.us i cc.tx.us i cc.ut.us i cc.vi.us i cc.vt.us i cc.va.us i cc.wa.us i cc.wi.us i cc.wv.us i cc.wy.us i lib.ak.us i lib.al.us i lib.ar.us i lib.as.us i lib.az.us i lib.ca.us i lib.co.us i lib.ct.us i lib.dc.us i lib.de.us i lib.fl.us i lib.ga.us i lib.gu.us i lib.hi.us i lib.ia.us i lib.id.us i lib.il.us i lib.in.us i lib.ks.us i lib.ky.us i lib.la.us i lib.ma.us i lib.md.us i lib.me.us i lib.mi.us i lib.mn.us i lib.mo.us i lib.ms.us i lib.mt.us i lib.nc.us i lib.nd.us i lib.ne.us i lib.nh.us i lib.nj.us i lib.nm.us i lib.nv.us i lib.ny.us i lib.oh.us i lib.ok.us i lib.or.us i lib.pa.us i lib.pr.us i lib.ri.us i lib.sc.us i lib.sd.us i lib.tn.us i lib.tx.us i lib.ut.us i lib.vi.us i lib.vt.us i lib.va.us i lib.wa.us i lib.wi.us i lib.wy.us i pvt.k12.ma.us i chtr.k12.ma.us i paroch.k12.ma.us i uy i com.uy i edu.uy i gub.uy i mil.uy i net.uy i org.uy i uz i co.uz i com.uz i net.uz i org.uz i va i vc i com.vc i net.vc i org.vc i gov.vc i mil.vc i edu.vc i ve i arts.ve i co.ve i com.ve i e12.ve i edu.ve i firm.ve i gob.ve i gov.ve i info.ve i int.ve i mil.ve i net.ve i org.ve i rec.ve i store.ve i tec.ve i web.ve i vg i vi i co.vi i com.vi i k12.vi i net.vi i org.vi i vn i com.vn i net.vn i org.vn i edu.vn i gov.vn i int.vn i ac.vn i biz.vn i info.vn i name.vn i pro.vn i health.vn i vu i com.vu i edu.vu i net.vu i org.vu i wf i ws i com.ws i net.ws i org.ws i gov.ws i edu.ws i yt i امارات i বাংলা i 中国 i 中國 i الجزائر i مصر i გე i 香港 i भारत i بھارت i భారత్ i ભારત i ਭਾਰਤ i ভারত i இந்தியா i ایران i ايران i الاردن i 한국 i қаз i ලංකා i இலங்கை i المغرب i мон i مليسيا i عمان i فلسطين i срб i пр.срб i орг.срб i обр.срб i од.срб i упр.срб i ак.срб i рф i قطر i السعودية i السعودیة i السعودیۃ i السعوديه i سورية i سوريا i 新加坡 i சிங்கப்பூர் i ไทย i تونس i 台灣 i 台湾 i 臺灣 i укр i اليمن i xxx i ye w za w zm w zw w онлайн i сайт i شبكة i 游戏 i 企业 i camera i clothing i lighting i singles i ventures i voyage i guru i holdings i equipment i bike i estate i tattoo i 在线 i 中文网 i land i plumbing i contractors i sexy i menu i 世界 i uno i gallery i technology i 集团 i reviews i guide i 我爱你 i graphics i construction i onl i みんな i diamonds i kiwi i enterprises i today i futbol i photography i tips i directory i kitchen i 移动 i kim i 삼성 i monash i wed i pink i ruhr i buzz i careers i shoes i موقع i career i otsuka i 中信 i gift i recipes i coffee i luxury i domains i photos i limo i viajes i wang i democrat i mango i cab i support i dance i nagoya i computer i wien i berlin i codes i email i بازار i repair i holiday i center i systems i wiki i ceo i international i solar i company i education i training i academy i marketing i florist i solutions i build i institute i builders i red i blue i ninja i business i gal i social i house i camp i immobilien i moda i glass i management i kaufen i farm i 公益 i 政务 i club i voting i tokyo i moe i guitars i bargains i 组织机构 i desi i cool i boutique i pics i орг i 公司 i 网络 i cheap i 广东 i photo i network i zone i 机构 i link i qpon i संगठन i agency i tienda i works i london i watch i rocks i shiksha i дети i budapest i nrw i vote i fishing i expert i horse i christmas i cooking i 商城 i casa i rich i voto i tools i 八卦 i praxi i events i flights i report i partners i neustar i rentals i catering i community i maison i parts i cleaning i okinawa i foundation i properties i vacations i productions i industries i haus i vision i mormon i cards i ink i villas i consulting i cruises i krd i xyz i dating i exposed i condos i eus i caravan i actor i saarland i yokohama i pub i рус i ren i fish i bar i dnp i bid i supply i miami i supplies i quebec i moscow i globo i axa i москва i 商店 i vodka i rest i frogans i wtc i rodeo i sohu i best i country i kred i feedback i work i luxe i ryukyu i autos i homes i jetzt i yachts i motorcycles i mini i ggee i beer i 佛山 i college i ovh i meet i 网址 i gop i blackfriday i lacaixa i 商标 i vegas i black i soy i trade i gent i ing i dad i shriram i bayern i scot i webcam i foo i eat i nyc i prod i how i day i meme i mov i paris i boo i new i ifm i life i archi i spiegel i brussels i church i here i dabur i vlaanderen i cologne i 手机 i wme i nhk i suzuki i whoswho i scb i hamburg i services i bzh i rio i cash i gives i hiphop i degree i digital i rehab i wtf i financial i limited i discount i fail i vet i ngo i fitness i schule i navy i bio i ong i town i toys i army i engineering i capital i exchange i fan i market i media i lease i university i reisen i airforce i pictures i gripe i engineering i associates i 政府 i williamhill i hiv i sca i reise i accountants i clinic i versicherung i top i furniture i dental i fund i creditcard i insure i audio i claims i loans i auction i attorney i finance i investments i juegos i dentist i lds i lawyer i surgery i gratis i software i mortgage i republican i credit i tax i africa i joburg i durban i capetown i sap i datsun i infiniti i firmdale i organic i nissan i website i space i schmidt i cuisinella i samsung i crs i doosan i press i emerck i erni i direct i yandex i lotto i toshiba i bauhaus i host i ltda i global i abogado i place i tirol i gmx i tatar i scholarships i eurovision i wedding i active i madrid i youtube i sharp i uol i physio i gmail i channel i fly i zip i esq i rsvp i wales i cymru i green i lgbt i 网店 i cancerresearch i everbank i frl i property i forsale i seat i deals i nra i 娱乐 i realtor i bnpparibas i melbourne i hosting i yoga i city i bond i click i cern i cloudfront.net i ap-northeast-1.compute.amazonaws.com i ap-southeast-1.compute.amazonaws.com i ap-southeast-2.compute.amazonaws.com i cn-north-1.compute.amazonaws.cn i compute.amazonaws.cn i compute.amazonaws.com i compute-1.amazonaws.com i eu-west-1.compute.amazonaws.com i sa-east-1.compute.amazonaws.com i us-east-1.amazonaws.com i us-gov-west-1.compute.amazonaws.com i us-west-1.compute.amazonaws.com i us-west-2.compute.amazonaws.com i z-1.compute-1.amazonaws.com i z-2.compute-1.amazonaws.com i elasticbeanstalk.com i elb.amazonaws.com i s3.amazonaws.com i s3-us-west-2.amazonaws.com i s3-us-west-1.amazonaws.com i s3-eu-west-1.amazonaws.com i s3-ap-southeast-1.amazonaws.com i s3-ap-southeast-2.amazonaws.com i s3-ap-northeast-1.amazonaws.com i s3-sa-east-1.amazonaws.com i s3-us-gov-west-1.amazonaws.com i s3-fips-us-gov-west-1.amazonaws.com i s3-website-us-east-1.amazonaws.com i s3-website-us-west-2.amazonaws.com i s3-website-us-west-1.amazonaws.com i s3-website-eu-west-1.amazonaws.com i s3-website-ap-southeast-1.amazonaws.com i s3-website-ap-southeast-2.amazonaws.com i s3-website-ap-northeast-1.amazonaws.com i s3-website-sa-east-1.amazonaws.com i s3-website-us-gov-west-1.amazonaws.com i betainabox.com i ae.org i ar.com i br.com i cn.com i com.de i com.se i de.com i eu.com i gb.com i gb.net i hu.com i hu.net i jp.net i jpn.com i kr.com i mex.com i no.com i qc.com i ru.com i sa.com i se.com i se.net i uk.com i uk.net i us.com i uy.com i za.bz i za.com i africa.com i gr.com i in.net i us.org i co.com i c.la i cloudcontrolled.com i cloudcontrolapp.com i co.ca i co.nl i co.no i cupcake.is i dreamhosters.com i dyndns-at-home.com i dyndns-at-work.com i dyndns-blog.com i dyndns-free.com i dyndns-home.com i dyndns-ip.com i dyndns-mail.com i dyndns-office.com i dyndns-pics.com i dyndns-remote.com i dyndns-server.com i dyndns-web.com i dyndns-wiki.com i dyndns-work.com i dyndns.biz i dyndns.info i dyndns.org i dyndns.tv i at-band-camp.net i ath.cx i barrel-of-knowledge.info i barrell-of-knowledge.info i better-than.tv i blogdns.com i blogdns.net i blogdns.org i blogsite.org i boldlygoingnowhere.org i broke-it.net i buyshouses.net i cechire.com i dnsalias.com i dnsalias.net i dnsalias.org i dnsdojo.com i dnsdojo.net i dnsdojo.org i does-it.net i doesntexist.com i doesntexist.org i dontexist.com i dontexist.net i dontexist.org i doomdns.com i doomdns.org i dvrdns.org i dyn-o-saur.com i dynalias.com i dynalias.net i dynalias.org i dynathome.net i dyndns.ws i endofinternet.net i endofinternet.org i endoftheinternet.org i est-a-la-maison.com i est-a-la-masion.com i est-le-patron.com i est-mon-blogueur.com i for-better.biz i for-more.biz i for-our.info i for-some.biz i for-the.biz i forgot.her.name i forgot.his.name i from-ak.com i from-al.com i from-ar.com i from-az.net i from-ca.com i from-co.net i from-ct.com i from-dc.com i from-de.com i from-fl.com i from-ga.com i from-hi.com i from-ia.com i from-id.com i from-il.com i from-in.com i from-ks.com i from-ky.com i from-la.net i from-ma.com i from-md.com i from-me.org i from-mi.com i from-mn.com i from-mo.com i from-ms.com i from-mt.com i from-nc.com i from-nd.com i from-ne.com i from-nh.com i from-nj.com i from-nm.com i from-nv.com i from-ny.net i from-oh.com i from-ok.com i from-or.com i from-pa.com i from-pr.com i from-ri.com i from-sc.com i from-sd.com i from-tn.com i from-tx.com i from-ut.com i from-va.com i from-vt.com i from-wa.com i from-wi.com i from-wv.com i from-wy.com i ftpaccess.cc i fuettertdasnetz.de i game-host.org i game-server.cc i getmyip.com i gets-it.net i go.dyndns.org i gotdns.com i gotdns.org i groks-the.info i groks-this.info i ham-radio-op.net i here-for-more.info i hobby-site.com i hobby-site.org i home.dyndns.org i homedns.org i homeftp.net i homeftp.org i homeip.net i homelinux.com i homelinux.net i homelinux.org i homeunix.com i homeunix.net i homeunix.org i iamallama.com i in-the-band.net i is-a-anarchist.com i is-a-blogger.com i is-a-bookkeeper.com i is-a-bruinsfan.org i is-a-bulls-fan.com i is-a-candidate.org i is-a-caterer.com i is-a-celticsfan.org i is-a-chef.com i is-a-chef.net i is-a-chef.org i is-a-conservative.com i is-a-cpa.com i is-a-cubicle-slave.com i is-a-democrat.com i is-a-designer.com i is-a-doctor.com i is-a-financialadvisor.com i is-a-geek.com i is-a-geek.net i is-a-geek.org i is-a-green.com i is-a-guru.com i is-a-hard-worker.com i is-a-hunter.com i is-a-knight.org i is-a-landscaper.com i is-a-lawyer.com i is-a-liberal.com i is-a-libertarian.com i is-a-linux-user.org i is-a-llama.com i is-a-musician.com i is-a-nascarfan.com i is-a-nurse.com i is-a-painter.com i is-a-patsfan.org i is-a-personaltrainer.com i is-a-photographer.com i is-a-player.com i is-a-republican.com i is-a-rockstar.com i is-a-socialist.com i is-a-soxfan.org i is-a-student.com i is-a-teacher.com i is-a-techie.com i is-a-therapist.com i is-an-accountant.com i is-an-actor.com i is-an-actress.com i is-an-anarchist.com i is-an-artist.com i is-an-engineer.com i is-an-entertainer.com i is-by.us i is-certified.com i is-found.org i is-gone.com i is-into-anime.com i is-into-cars.com i is-into-cartoons.com i is-into-games.com i is-leet.com i is-lost.org i is-not-certified.com i is-saved.org i is-slick.com i is-uberleet.com i is-very-bad.org i is-very-evil.org i is-very-good.org i is-very-nice.org i is-very-sweet.org i is-with-theband.com i isa-geek.com i isa-geek.net i isa-geek.org i isa-hockeynut.com i issmarterthanyou.com i isteingeek.de i istmein.de i kicks-ass.net i kicks-ass.org i knowsitall.info i land-4-sale.us i lebtimnetz.de i leitungsen.de i likes-pie.com i likescandy.com i merseine.nu i mine.nu i misconfused.org i mypets.ws i myphotos.cc i neat-url.com i office-on-the.net i on-the-web.tv i podzone.net i podzone.org i readmyblog.org i saves-the-whales.com i scrapper-site.net i scrapping.cc i selfip.biz i selfip.com i selfip.info i selfip.net i selfip.org i sells-for-less.com i sells-for-u.com i sells-it.net i sellsyourhome.org i servebbs.com i servebbs.net i servebbs.org i serveftp.net i serveftp.org i servegame.org i shacknet.nu i simple-url.com i space-to-rent.com i stuff-4-sale.org i stuff-4-sale.us i teaches-yoga.com i thruhere.net i traeumtgerade.de i webhop.biz i webhop.info i webhop.net i webhop.org i worse-than.tv i writesthisblog.com i a.ssl.fastly.net i b.ssl.fastly.net i global.ssl.fastly.net i a.prod.fastly.net i global.prod.fastly.net i firebaseapp.com i github.io i githubusercontent.com i ro.com i appspot.com i blogspot.be i blogspot.bj i blogspot.ca i blogspot.cf i blogspot.ch i blogspot.co.at i blogspot.co.il i blogspot.co.nz i blogspot.co.uk i blogspot.com i blogspot.com.ar i blogspot.com.au i blogspot.com.br i blogspot.com.es i blogspot.cv i blogspot.cz i blogspot.de i blogspot.dk i blogspot.fi i blogspot.fr i blogspot.gr i blogspot.hk i blogspot.hu i blogspot.ie i blogspot.in i blogspot.it i blogspot.jp i blogspot.kr i blogspot.mr i blogspot.mx i blogspot.nl i blogspot.no i blogspot.pt i blogspot.re i blogspot.ro i blogspot.se i blogspot.sg i blogspot.sk i blogspot.td i blogspot.tw i codespot.com i googleapis.com i googlecode.com i withgoogle.com i herokuapp.com i herokussl.com i iki.fi i biz.at i info.at i co.pl i azurewebsites.net i azure-mobile.net i cloudapp.net i nfshost.com i nyc.mn i nid.io i operaunite.com i outsystemscloud.com i rhcloud.com i priv.at i za.net i za.org i);

# Right-hand side of a domain name:
sub _rhs {
    my $domain = shift;
    return substr($domain, index($domain, '.') + 1);
}

sub _find_rule {
    my $domain = shift;
    my $rhs = _rhs($domain);
    my $rule = $rules{$domain};

    return do {
        # Test for rule match with full domain:
        if ( defined $rule ) {
            # An identity rule match means the full domain is the public suffix:
            if ( $rule eq 'i' ) { $domain } # return undef in scalar context

            # Fail out if a wilcard rule matches the full domain:
            elsif ( $rule eq 'w' ) { () }

            # An exception rule means the right-hand side is the public suffix:
            else { $rhs }
        }

        # Fail out if no match found and the full domain and right-hand side are
        # identical:
        elsif ( $domain eq $rhs ) { () }

        # No match found with the full domain, but there are more levels of the
        # domain to check:
        else {
            my $rrule = $rules{$rhs};

            # Test for rule match with right-hand side:
            if (defined $rrule) {

                # If a wildcard rule matches the right-hand side, the full
                # domain is the public suffix:
                if ( $rrule eq 'w' ) { $domain }

                # An identity rule match means it's the right-hand side:
                elsif ( $rrule eq 'i' ) { $rhs }

                # An exception rule match means it's the right-hand side of the
                # right-hand side:
                else { _rhs($rhs) }
            }

            # Try again with the right-hand side as the full domain:
            else {
                _find_rule($rhs);
            }
        }
    }
}

1;
=encoding utf8

=head1 NAME

Mozilla::PublicSuffix - Get a domain name's public suffix via the Mozilla Public Suffix List

=head1 SYNOPSIS

    use feature qw(say);
    use Mozilla::PublicSuffix qw(public_suffix);

    say public_suffix('org');       # 'org'
    say public_suffix('perl.org');  # 'org'
    say public_suffix('perl.orc');  # undef
    say public_suffix('ga.gov.au'); # 'gov.au'
    say public_suffix('ga.goo.au'); # undef

=head1 DESCRIPTION

This module provides a single function that returns the I<public suffix> of a
domain name by referencing a parsed copy of Mozilla's Public Suffix List.
From the official website at L<http://publicsuffix.org/>:

=over

A "public suffix" is one under which Internet users can directly register names.
Some examples of public suffixes are com, co.uk and pvt.k12.wy.us. The Public
Suffix List is a list of all known public suffixes.

=back

A copy of the official list is bundled with the distribution. As the official
list continues to be updated, the bundled copy will inevitably fall out of date.
Aside from new releases always including the latest version of the list, this
distribution's installer provides the option (defaults to "No") to check for a
new version of the list and download/use it if one is found.

=head1 FUNCTIONS

=over

=item public_suffix($domain)

Exported on request. Simply returns the public suffix of the passed domain name,
or C<undef> if either the domain name is not well-formed or the public suffix is
not found.

=back

=head1 SEE ALSO

=over

=item L<Domain::PublicSuffix>

Similar to this module, with an object-oriented interface and somewhat
alternative interpretation of the rules Mozilla stipulates for determining a
public suffix.

=back

=head1 AUTHOR

Richard Simões C<< <rsimoes AT cpan DOT org> >>

=head1 COPYRIGHT & LICENSE

Copyright © 2013 Richard Simões. This module is released under the terms of the
B<MIT License> and may be modified and/or redistributed under the same or any
compatible license.
